#pragma rtGlobals=3		// Use modern global access method.
#include <All IP Procedures>




// *************************************************************************************************
// -------------------------------------	some global constants   ------------------------------------------
// *************************************************************************************************

StrConstant version = "iSpectra version 1.02"

// -------------------------------------	default elements and emission energies  -------------------------------

StrConstant elements = "C;O;F;Na;Mg;Al;Si;P_k;S_k;Cl;K;Ca;Ti;V;Cr;Mn;Fe;Co;Ni;"
StrConstant energies = "0.282;0.526;0.677;1.04;1.25;1.49;1.74;2.02;2.31;2.62;3.31;3.69;4.51;4.95;5.41;5.9;6.4;6.93;7.48;"  // in keV, same order as element above

// -------------------------------------	numeric constants  ---------------------------------------------

static constant avgChannels =    3       				// set the number of channels to be averaged at given element position; function AnalyseCube()
static constant xPerChannel = 0.01					//default x- (energy) spacing of cube in keV; set by spectral imaging system; 0.01 for Thermo system 
static constant OverVoltage = 1.5         			   	// Overvoltage factor; e.g. function FindGroups() will ignore k-lines above 10 kV when spectrum is aquired at 15 kV
static constant maxPhases = 100					// function FindGroups(); prevents unrealistic number of goups in automated search
static constant MinPhaseFraction = 0.5				// minimum pixel percent abundance under which a phase is marked red on iSpectraGroups panel
static constant MaxkeVSpecVal = 8					// max keV range shown in displayed spectra; 
static constant minParticleArea = 9					// minimum number of pixels above which a group is marked green when running runParticleAnalysis()
static constant nChannelsForCrossCorrelation = 650  // number of channels that is used for cross correlation, lower is faster, higher is slower but more accurate
static constant IntervalForCrossCorrelation = 3 		// sampling interval for cross correlation. Higher is faster; only every i'th channel is considered
static constant HTMLimageWidth = 640				// width of images on HTML output in pixels

// ---------SEM magnification calibration function: ImageWidth (in microns) = 10^(MagCal_A * log(Magnification) + MagCal_B) ------
// values valid for Jeol JSM-6390 LA  at Department of Earth Sciences, ETH Zurich, at 10 mm working distance

static constant MagCal_A = -0.999	 
static constant MagCal_B = 5.093


// *************************************************************************************************
// ---------------------------------  Menu     ---------------------------------------------------------
// *************************************************************************************************



Menu "iSpectra"						// build menu
		"Load Raw File", iS_LoadDataSet()
		"Load SEM TIF Image", iS_LoadTIF()
		"Re-Extract Element Maps", iS_ReAnalyseCube()
		"Model Setup", iS_ModelPanel()
		"-----------"
		Submenu "Element Energies"
			"Show Element Reference Lines", iS_ShowLineRef()
			"Save Line Reference Wave", iS_SaveLineRef()
			"Load Line Reference Wave", iS_LoadLineRef()
			"Sort Line Reference Wave", iS_SortLineRef()
		end
		"-----------"
		"Add Line References to Top Trace", iS_AddLineRefToTrace()
		"Find Particles", iS_runParticleAnalysis()
		"Show Intensity Ternary", iS_IntensityTernary()
		"Show Base Image", iS_DisplayBaseImage()
		"Show Group Panel", iS_iSpectraGroupPanel()
		"Show Group Image", iS_OpenGroupsOnGrey()
		"Create Gradient ", iS_PlotGradient()
		"Correct Phase Morphology", iS_GroupForImageMorphology()
		"Single Phase Analysis", iS_PromptForSinglePhaseAnalysis()
		"-----------"
		"Create Restore Point", iS_PromptForRestorePoint()
		"Load Restore Point", iS_LoadRestorePanel()
		"-----------"
		SubMenu "Export"
			"Export Selected Spectra", iS_ExportSpectra(0)
			"Export all Spectra", iS_ExportSpectra(1)		
			"Full Export", iS_FullSave()
		end
		"-----------"
		SubMenu "History Actions"
			"Show History", iS_ShowHistory()
			"Save History", iS_SaveHistory()
			"Apply History", iS_ApplyHistory()
			"Setup Batch Processing", iS_SetupBatchProcessing()
			"Start Batch Processing", iS_runBatchFromMenu()
			"Create Montage",  iS_LoadImagesForStitching()
			"Make Legend from Batch Summary", iS_MakeReportFromBatchSummary(BatchSummary)
		end
		"-----------"
		SubMenu "Colour Data Base"
			"Create New Colour Data Base", iS_MakeNewColourDB()
			"Load or Show Data Base",  iS_LoadOrShowColourDB()
			"Save Data Base",  iS_SaveColourDB()
		end
		"-----------"
		"Erase Dataset", iS_clearAll()
		"-----------"
		"About iSpectra", iS_About()
end

// *************************************************************************************************
// ----------some small tool functions ---------------------------------------------------
// *************************************************************************************************

function iS_about()
	DoAlert /T="About iSpectra"0, version  + "\r" +  "copyright C. Liebske\r "
end


Function iS_SaveLineRef()
	wave/Z LineRef = root:packages:ispectra:LineRef
	
	if(!WaveExists(LineRef))
		DoAlert 0, "WaveExists() Error."
		return 0
	endif
	
	Save LineRef	
end


function iS_LoadLineRef()
	
	if(!DataFolderExists("root:packages:ispectra:"))
		return 0
	endif
	
	if(!WaveExists(root:cube))
		DoAlert 0, "WaveExists() Error."
		return 0
	endif
	
	// load LineRef
	LoadWave
	
	String name = RemoveEnding(S_WaveNames, ";")
	wave NewLineRef = $name
	
	// replace LineRef
	Duplicate/O NewLineRef root:Packages:iSpectra:LineRef
	KillWaves/Z NewLineRef
	
	// re-analyse cube
	iS_AnalyseCube(:cube, :packages:ispectra:LineRef, 3)
end



// ----------kill everything within the experiment---------------------------------------------------

function iS_clearAll()
	
	DoAlert 1, "This will erase the entire dataset! Continue?"
	if(V_flag==2)
		return 2
	endif
	
	variable i
	string ThisOne =""
	
	// get lists of windows, waves, and datafolders
	string windows = WinList("*", ";", "WIN:67")
	string DataFolders = StringByKey("FOLDERS", DataFolderDir(1))
	
	// kill all windows
	do
		ThisOne = StringFromList(i, windows)
		if (strlen(ThisOne)==0)
			break
		endif
		KillWindow $ThisOne
		i+=1
	while(1)
		
	// Kill all datafolders
	i=0
	do
		ThisOne = StringFromList(i, DataFolders, ",")
		if (strlen(ThisOne)==0)
			break
		endif
		KillDataFolder ThisOne
		i+=1
	while(1)
	
	//Kill all waves
	KillWaves/A/Z
	
	// Kill all paths
	KillPath/A/Z
	
	return 1
end


// *************************************************************************************************
// ----------			Particle analysis functions    				-----------------------------------------------
// *************************************************************************************************


function iS_runParticleAnalysis()
	
	wave/Z M_Groups = Root:M_Groups
	wave/Z/T M_Model = Root:packages:ispectra:M_Model
	
	if(!WaveExists(M_Groups) || !WaveExists(M_Model))
		DoAlert 0, "WaveExists() Error."
		return 0
	endif
	
	DoWindow/F iSpectraGroups
	if(!V_Flag)
		return 0
	endif
	
	string dfsave = GetDataFolder(1)
	
	variable i
	for(i=0; i<DimSize(M_Model, 0); i+=1)
		
		variable PixPercent = str2num(M_Model[i][%PixPercent])
		
		// if group is abundant to certain percentage, skip it
		if (PixPercent >= MinPhaseFraction)
			continue
		endif
		
		// run particle analysis; return value is 1 for particle 0 for no particle
		variable WhatIsIt = iS_isParticle(i, minParticleArea)
		
		
		// mark background colour on GroupPanel green
		if(WhatIsIt)
			SetVariable $"PixPerc" + num2Str(i),  win = iSpectraGroups,  valueBackColor=(0, 65535,  0 )
		endif
		
	endfor
end


function iS_isParticle(group, minParticleArea)
	variable group, minParticleArea
	
	//return value is 1 for group being a particle or 0 for no particle
	
	wave/Z M_Groups = Root:M_Groups
	wave/Z/T M_Model = Root:packages:ispectra:M_Model
	
	if(!WaveExists(M_Groups) || !WaveExists(M_Model))
		Abort "WaveExists() Error."
	endif
	
	variable returnval = 0
	
	string dfsave = GetDataFolder(1)

	NewDataFolder/O/S ImageTmp
		
	Duplicate/O M_Groups $"Group_" + num2str(group)
	wave w = $"Group_" + num2str(group)
	
	MultiThread w = M_Groups == group ? 0 : 255
		
	Redimension/B/U w
	ImageAnalyzeParticles /E/W/Q/M=3/A=3 stats, w
	wave W_ImageObjArea
	
	variable maxArea = WaveMax(W_ImageObjArea)
		
	if (maxArea >= minParticleArea)
		returnval = 1
	endif 
		
	SetDatafolder dfsave
	KillDataFolder/Z ImageTmp	

	return returnval
end




// ----------add elements to top trace  --------------------------------------------------



function iS_AddLineRefToTrace()
	
	String List = winlist("*spectrum*", ";", "WIN:1")
	if(strlen(list) == 0)
		return 0
	endif
	
	string TopGraph = StringFromList(0, List)
	iS_PutMarkerOnTopTrace(TopGraph)	
end


Function iS_PutMarkerOnTopTrace(graphName)
	String graphName
	
	DoWindow/F $GraphName
	if(!V_flag)
		return 0
	endif	
	
	DFREF dfr = root:packages:ispectra:
	wave/Z LineRef = dfr:LineRef
	
	if(!WaveExists(LineRef))
		DoAlert 0, "WaveExists() Error."
		return 0
	endif
	
	string List = TraceNameList(graphname, ";", 1)
	variable numItems = ItemsInList(List, ";")
	string topTrace = StringFromList(numItems-1, List)
	wave w = TraceNameToWaveRef(graphName, topTrace)
	
	wave/T/Z Labels = dfr:ElementLabels
	
	if(!WaveExists(Labels))
	
		Make/O/T/N=(numPnts(w)) dfr:ElementLabels
		wave/T Labels = dfr:ElementLabels
		SetScale/I x, 0, 15, "keV", Labels
		variable i, xx
		String thisElement = ""
		for(i=0; i<numPnts(lineRef); i+=1)
			xx = LineRef[i]/xPerChannel
			Labels[xx] = GetDimLabel(LineRef, 0, i)
		endfor 
		
	endif
	
	ModifyGraph msize($topTrace)=3, mode($topTrace) = 4
	ModifyGraph/W=$graphName textMarker($topTrace)={Labels,"default",0,0,5,0.00,25}
end


function iS_LoadTIF()
	wave/Z M_SumImage
	
	if(!WaveExists(M_SumImage))
		DoAlert 0, "You must first load SI data!"
		return 0
	endif
	
	variable Rows = DimSize(M_SumImage, 0)
	variable cols = DimSize(M_SumImage, 1)
	
	ImageLoad/C=1/Q/T=any
	
	if (!V_flag)
		return 0
	endif
	
	string wNameString = StringFromList(0, S_WaveNames)
	string MapName = StringFromList(1, S_WaveNames)
	
	KillWaves/Z $MapName
	
	wave M_Image = $wNameString
	
	// if image is SIREF it may require downsampling to match SI-resolution 
	variable xScale = round(DimSize(M_Image, 0)/rows)
	variable yScale = round(DimSize(M_Image, 1)/cols)
	ImageInterpolate /DEST=M_TIFImage/PXSZ={xScale, yScale} Pixelate M_Image
	
	
	// Make image 16-bit and scale grey scale values
	wave M_TifImage=root:M_TifImage
	Redimension/U/W M_TifImage
	iS_ScaleImageTo16bit(M_TifImage)
	iS_ScaleIfMagIsKnown(M_TifImage)
	
	KillWaves/Z M_Image
end



function iS_ShowLineRef()

	wave/Z LineRef = root:packages:iSpectra:LineRef	
	if (!WaveExists(LineRef) )
		DoAlert 0, "WaveExists() Error."
		return 0
	endif
	
	edit/K=1 :packages:iSpectra:LineRef.ld
end

function iS_SortLineRef()

	wave/Z LineRef = root:packages:iSpectra:LineRef	
	if (!WaveExists(LineRef) )
		DoAlert 0, "WaveExists() Error."
		return 0
	endif
	
	Sort/DIML root:packages:iSpectra:LineRef root:packages:iSpectra:LineRef 
	
	wave/Z Cube = root:Cube
	if(WaveExists(cube))	
		iS_AnalyseCube(:cube, LineRef, 3)
	endif
end

function iS_ShowHistory()
	
	DFREF dfr = root:packages:iSpectra
	wave/T/Z History= dfr:History
	if (!WaveExists(History) )
		DoAlert 0, "WaveExists() Error."
		return 0
	endif
	
	edit/K=1 dfr:History
	ModifyTable alignment(dfr:History)=0, width (dfr:History) = 300
end



function iS_ReAnalyseCube()
	wave/Z cube = root:cube
	wave/Z LineRef = root:packages:iSpectra:LineRef
	
	if (!WaveExists(cube) ||  !WaveExists(LineRef) )
		DoAlert 0, "WaveExists() Error."
		return 0
	endif

	variable nChannels
	Prompt nChannels, "Enter Channels to be averaged for element maps (default is 3)"
	DoPrompt "Enter number of channels",nChannels
	
	iS_AnalyseCube(cube, lineref, nChannels)
end



function iS_OpenGroupsOnGrey()
	wave/Z M_Groups = root:M_Groups
	wave/Z M_BaseImage =  root:packages:iSpectra:M_BaseImage
	wave/Z M_Model = root:packages:iSpectra:M_Model
	
	if (!WaveExists(M_Groups) ||  !WaveExists(M_BaseImage)  ||  !WaveExists(M_Model) )
		DoAlert 0, "WaveExists() Error."
		return 0
	endif
	
	iS_DisplayGroupsOnGrey(M_Model, M_Groups, M_BaseImage)
end


function iS_DisplayBaseImage()
	
	wave/Z M_BaseImage = root:packages:ispectra:M_BaseImage
	if(!WaveExists(M_BaseImage))
		return 0
	endif
			
	DoWindow BaseImage
	if(V_flag)
		return 1
	endif
	
	NewImage/K=1/N=BaseImage M_BaseImage
	SetWindow BaseImage,hook(s)=iS_Hook
	
	return 2
end


Function iS_BaseImageSelectorProc(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	
	wave/Z M_BaseImage = root:packages:ispectra:M_BaseImage
	if(!WaveExists(M_BaseImage))
		return 0
	endif
	
	wave/Z M_Image = $popstr
	If(!WaveExists(M_Image))
		Abort "No such image!"
	endif
	
	if(DimSize(M_Image,2) == 2)
		M_BaseImage=M_Image
	else
		M_BaseImage[][][0,2] = M_Image[p][q]
	endif
	
	
	//Display if not done yet
	DoWindow BaseImage
	if(V_flag ==0)
		iS_DisplayBaseImage()
	endif
	
	return 1
End

function iS_DisplayGroupsOnGrey(M_Model, M_Groups, M_BaseImage)
	Wave/T M_Model 
	wave M_groups, M_BaseImage
	
	DoWindow/F BaseImage
	if (V_Flag)
		// is M_Groups already on - if not add
		string List = ImageNameList("BaseImage", ";")
		if(strsearch(list, "M_Groups",0) == -1)
			AppendImage M_Groups
		endif
	endif
	
	if (V_flag==0)
		iS_DisplayBaseImage()
		AppendImage M_Groups
	endif
		
	variable i, R, G, B
	
	for(i=0; i<DimSize(M_Model, 0); i+=1)
		
		if(str2num(M_Model[i][%Show]) == 0)
			ModifyImage M_groups explicit = 1, eval={i,-1,-1,-1}	
			continue
		endif

		r = str2num(M_Model[i][%R])
		g =  str2num(M_Model[i][%G])
		b =  str2num(M_Model[i][%B])
	
		ModifyImage M_groups explicit = 1, eval={i,r,g,b}		
	endfor
end


// *************************************************************************************************
// --------------------------------- Initialise iSpectra   -------------------------------------------------
// *************************************************************************************************


function iS_Initialise()		//Make globals 

	if (DataFolderExists("root:Packages:iSpectra:"))
		return 0
	endif
	
	string dfsave = GetDataFolder(1)
	
	NewDataFolder/O root:Packages
	NewDataFolder/O/S root:Packages:iSpectra	
	
	
	// make a settings wave for global variables
	Make/O/N=14 Settings	
	
	SetDimLabel 0,0, start, 				Settings
	SetDimLabel 0,1, thresh, 				Settings
	SetDimLabel 0,2, offset, 				Settings
	SetDimLabel 0,3, nIntPlots,				Settings
	SetDimLabel 0,4, Magnification,			Settings  
	SetDimLabel 0,5, MinPix, 				Settings
	SetDimLabel 0,6, Erosion, 				Settings
	SetDimLabel 0,7, vCorr, 				Settings
	SetDimLabel 0,8, AccV, 	 				Settings
	SetDimLabel 0,9, R, 	 				Settings
	SetDimLabel 0,10, G, 	 				Settings
	SetDimLabel 0,11, B, 	 				Settings
	SetDimLabel 0,12, ErosionMode,		Settings
	SetDimLabel 0,13, DwellTimePerPixel,	Settings
	
	// set some default values	
	Settings[%Erosion] = 0
	Settings[%ErosionMode] = 6
	Settings[%nIntPlots] = 0
	Settings[%vCorr] = 0.99
	
	string /g xElem 
	string /g yElem 
	string /g ActiveWindow="ImageZoom" 
	string /g ActivePhases
	String /g CubeFileName
	
	
	// set up a LineRef wave	 with static constants
		
	if (ItemsInlist(energies) != ItemsInList(elements) )
		DoAlert 0, "Mismatch between default elements and reference energies!"
		SetDataFolder dfsave
		return 0
	endif
	
	Make/O/N=(ItemsInList(energies)) LineRef
	LineRef = str2num(StringFromList(p, energies))
	
	variable i
	for(i=0; i<numpnts(LineRef); i+=1)
		SetDimLabel 0, i, $StringFromList(i, elements), LineRef	
	endfor
	Sort/DIML LineRef LineRef
	
	// make a History wave to record everything done to the dataset
	Make/T/O/N=1 History
	History[0] = "iSpectraHISTORY:"
	
	SetDataFolder dfsave
	
	return 1
end


// *************************************************************************************************
// ---------------------------------  Import and Analyse -------------------------------------------------
// *************************************************************************************************



function iS_LoadDataSet()

	if(!DataFolderExists("root:packages:ispectra"))
		iS_Initialise()
	endif
	
	dfRef dfr =  root:Packages:iSpectra:	
	wave/z Settings = dfr:Settings
	Wave/T/Z History = dfr:History
	variable AccV, TimePerFrame, nFrames, Magnification
	
	if(!WaveExists(Settings) || !WaveExists(History))
		DoAlert 0, "WaveExists() Error."
		return 0
	endif
	

	// get rpl filepath 	
	String RPLFilePath =  iS_GetOneRPLFilePath(0)
	if(strlen(RPLFilePath)==0)
		return 0
	endif
	
	// load spectral cube (raw-file)
	iS_ReadLispixFilePair(RPLFilePath)
	wave/Z cube = root:cube
	if(!WaveExists(cube))
		DoAlert 0, "WaveExists() Error."
		return 0
	endif
	
	
//	// prompt for Acceleration Voltage, Time per frame and number of frames, and magnification
	Prompt AccV, "Enter Acceleration Voltage used to aquire Spectral Image (kV)"
	Prompt TimePerFrame, "Time per frame (s)"
	Prompt nFrames, "Number of Frames"
	Prompt Magnification, "Magnification. Leave 0 if not known or pixel size not calibrated!"
	DoPrompt "Enter SI acquisition parameters", AccV, TimePerFrame, nFrames, Magnification
	if(V_flag)		//user clicked cancel
		KillDataFolder/Z root:packages:iSpectra
		return 0	
	endif 
	
	Settings[%AccV] = AccV
	
	// calculate measurement time per pixel	
	Settings[%DwellTimePerPixel] = TimePerFrame * nFrames	/ (DimSize(cube, 0) * DimSize(cube, 1))
	Settings[%Magnification] = Magnification
	
	
	//write properties to History
	string cmd =""
	sprintf cmd, "AccV:%g", AccV
	iS_WriteString2History(cmd)
	
	sprintf cmd, "TimePerPix:%g", Settings[%DwellTimePerPixel]
	iS_WriteString2History(cmd)
	
	sprintf cmd, "Magnification:%g", Settings[%Magnification]
	iS_WriteString2History(cmd)
	
		
	
	//Clip unused channels
	DoAlert 1, " Do you want to clip unused channels (reduces file size)?"
	if (V_flag==1)
		iS_ClipChannels()
	endif	
	
	// get element intensities
	iS_AnalyseCube(:cube, :Packages:iSpectra:LineRef, avgChannels) 	
	
	//scale images and display display SumImage
	wave/Z M_SumImage = root:M_SumImage
	wave/Z M_BaseImage = dfr:M_BaseImage
	if (Magnification != 0)	
		iS_ScaleIfMagIsKnown(M_SumImage)
		iS_ScaleIfMagIsKnown(M_BaseImage)
	endif
	
	iS_DisplayBaseImage()
	
	// Open model panel
	iS_ModelPanel()
	
	return 1
end
	


function/S iS_GetOneRPLFilePath(Mult)
	variable Mult     // 0 or 1 for single file or multiple files
	variable refNum	
		
	Open/D/R/T=".rpl"/M="Select (a) RPL file(s)"/MULT=(Mult) refNum
	string filePath = S_fileName
	if( strlen(filePath) == 0 )
		print "user cancelled."	
	endif
	
	return filePath
end
 
 
 function iS_ReadLispixFilePair(ripplePath)
	string ripplePath		
	
  	variable timeref = StartMsTimer
  	
  	// need package folder - does it exist?
  	if(!DataFolderExists("root:packages:ispectra"))
		iS_Initialise()
	endif
  	 
   	
// 1. make sure there is an associated raw file, get its name
	string rawPath = ripplePath[0,strlen(ripplePath)-3] + "aw"
	GetFileFolderInfo/Q/Z rawPath
	if( V_Flag != 0 )
		print "no raw file found for "+ParseFilePath(0, ripplePath, ":", 1, 0)
	else
 
// 2. read ripple file, convert to keyword-value string
		variable refNum
		Open/R refNum as ripplePath
		string info = ""
		do
			string line
			FReadLine refNum, line
			if( strlen(line) == 0 )	// end of file
				break	// exit do-while loop
			endif
			if( !cmpstr(line[0],";") )	// comment line
				continue	// skip
			endif
			line = ReplaceString(" ", LowerStr(line), "")	// remove spaces and convert to lower case
			string keyStr, valStr
			sscanf line, "%s %s", keyStr, valStr
			info += keyStr + ":" + valStr + ";"	// convert to Igor-style keyword-value string
		while( 1 )	// exit is through break statement
		Close refNum
 
// 3. extract info from keyword-value string
		variable offset = NumberByKey("offset", info), byteOrder = 0
		if( !cmpstr(StringByKey("byte-order", info), "little-endian") )
			byteOrder = 1
		endif
		variable igorType = 0, length = NumberByKey("data-length", info)
		string dataType = StringByKey("data-type", info)
		strswitch(dataType)
			case "float":
				if( length == 4 || length == 8 )
					igorType = length/2
				endif
				break
			case "unsigned":
				igorType = 64
				// fall through
			case "signed":
				if( length == 1 || length == 2 || length == 4 )
					igorType += 8*length
				else
					igorType = 0
				endif
				break
		endswitch
		if( igorType == 0 )
			abort "illegal data type/length specification: "+dataType+"/"+num2str(length)
		endif
 
// 4. load raw file

		GBLoadWave /A=lispix /B=(byteOrder) /Q /S=(offset) /T={igorType,igorType} rawPath
 
		variable stoptime = (stopMStimer(timeref)/1e6)
 		timeref = StartMsTimer
 		
// 5. report to history and rename wave to filename
		if( V_flag == 0 )
			printf "no waves"
		else
			printf "Cube "
		endif
		print "loaded from "+S_path+S_fileName+"."
		if( V_Flag == 1 )	// should be one
		
			string oldName = StringFromList(0, S_waveNames)
			string newName = ParseFilePath(3, ripplePath, ":", 0, 0)
			wave oldWave = $oldName
			
			//Make Cube
			Duplicate/O OldWave, Cube
			wave Cube
			
		//	SVAR CubeFileName = root:Packages:iSpectra:CubeFileName
		//	CubeFileName = CleanUpName(NewName,0)
 
// 6. redimension data cube
			variable width = NumberByKey("width", info)
			variable height = NumberByKey("height", info)
			variable depth = NumberByKey("depth", info)
			Redimension/N=(width, height, depth) Cube
			
			// reassign cube data
			MultiThread Cube= oldWave[(p + q*width)*depth + r]
			
			// set z-scaling of cube
			SetScale/P z 0, xPerChannel, "kV", Cube
			
 // 7. get sum Image 
 
			MatrixOP/O M_SumImage = sumBeams(Cube)
			variable totCount = sum(M_SumImage)
			printf "Total number of counts: %g \r", totCount
			printf "Average counts per pixel: %g \r", totCount/(width*height)
			
			//scale image to 16 bit
			wave M_SumImage = root:M_SumImage
			Redimension/U/W M_SumImage
			iS_ScaleImageTo16bit(M_SumImage)
							
			// make a 3 layer RGB copy of M_SumImage as base image
			Make/O/W/U/N=(width, height, 3) root:packages:ispectra:M_BaseImage
			Wave M_BaseImage = root:packages:ispectra:M_BaseImage
			M_BaseImage[][][0,2] = M_SumImage[p][q]
					
//8. Clean up			
			KillWaves/Z oldWave
 
		endif
	endif
	
	Printf "Time required to load raw file: %f seconds\r", stoptime
	Printf "Time required to make 3D array from raw file: %f seconds\r", (stopMStimer(timeref)/1e6)	
end
	
function iS_ScaleImageTo16bit(M_Image)
	wave M_Image
	
	variable delta = 1/(WaveMax(M_Image)-WaveMin(M_Image))*65535
	variable DeltaM = delta * WaveMin(M_Image)
	
	Multithread M_Image = delta*M_Image-deltaM
	
	return 1
end	
	
	
	
function iS_ScaleIfMagIsKnown(M_Image)
	wave M_Image
	
	wave/Z Settings = root:packages:iSpectra:Settings
	variable Mag = Settings[%Magnification]
	
	if(!WaveExists(Settings) || Mag == 0)
		//Set default scale in case it has been modified
		SetScale/P x, 0, 1, "", M_Image
		SetScale/P y, 0, 1, "", M_Image
		return 1
	endif
	
	variable PixWidth, Microns, pixDimDelta
		
	PixWidth = DimSize(M_Image, 0)
	Microns = 10^(MagCal_A * log(Mag) + MagCal_B)  //calibrated on Image WIDTH
	pixDimDelta = Microns/PixWidth
	
	SetScale/P x, 0, pixDimDelta, "m", M_Image
	SetScale/P y, 0, pixDimDelta, "m", M_Image
	
	return pixDimDelta
end	
	
	
	

function iS_ClipChannels()
	Wave/Z cube = root:Cube
	dfRef dfr =  root:Packages:iSpectra:	
	wave/z Settings = dfr:Settings
	variable AccV = Settings[%AccV]
	Wave/T/Z History = dfr:History
	
	if(!WaveExists(cube) || !WaveExists(Settings) || !WaveExists(History) )
		DoAlert 0, "WaveExists() Error."
		return 0
	endif
	
	variable maxChannel = DimSize(cube, 2)
	variable LastUsedChannel = AccV / xPerChannel
	variable clippedChannels = maxChannel - LastUsedChannel
	
	if (LastUsedChannel > maxChannel)
	
		DeletePoints/M=2 (LastUsedChannel), (clippedChannels), cube	
	
		Printf "Initial number of channels: %g\r", maxChannel
		Printf "Number of clipped channels: %g\r", clippedChannels
		Printf "Number remaining channels: %g\r", DimSize(cube, 2)
	
		// Write to History
		iS_WriteString2History("CMD:iS_ClipChannels()")
		
		return 1
	endif
	
	return 0
end



function iS_AnalyseCube(cube, LineRef, avgChannels)  
	wave cube, LineRef	
	variable avgChannels        // averages number channels at given element position
	
	dfRef dfr =  root:Packages:iSpectra:	
	wave/Z LineRef = dfr:LineRef
	wave/T/Z History = dfr:History
	wave/Z Settings = dfr:Settings
	
	if(!WaveExists(cube) || !WaveExists(LineRef) || !WaveExists(History) || !WaveExists(Settings))
		DoAlert 0, "WaveExists() Error."
		return 0
	endif
	
	// kill folder if exists
	KillDataFolder/Z "root:Intensities:"
			
	variable timeref = startMSTimer
	
	//Make sure each point in cube is >0; the opposite may happen after background substraction
	//First check if bkg wave is present at all
		
	wave/Z bkg = root:bkg
	if (WaveExists(bkg))
		if (WaveMin(Cube) < 0)
			MultiThread cube = cube < 0 ? 0 : cube
		endif
	endif
		
	variable maxVoltage = Settings[%AccV]

	string dfsave = GetDataFolder(1)
	NewDataFolder/O/S Intensities
	
	String Element
	variable i, plane, range
	
	for(i=0; i<numpnts(LineRef); i+=1)
		
		Element = GetDimLabel(LineRef, 0, i)
		
		if (LineRef[i] >= MaxVoltage)
			break
		endif 
		
		//get plane from channel spacing
		
		plane = round(LineRef[i] / xPerChannel)
		range = trunc(avgChannels/2)
	
		Duplicate/FREE/O/R = [][][(plane-range), (plane + range)] cube tmp3D	
		MatrixOP/FREE avg = sumBeams(tmp3D) / avgChannels
		
		Multithread avg = round(avg)  //make integers from average values
		
		Duplicate/O avg $Element
		wave M_Map = 	$Element
		Redimension/W/U M_Map
		iS_ScaleIfMagIsKnown(M_Map)
				
	endfor
	
	KillWaves/Z M_ImagePlane
	SetDataFolder $dfsave
	
	// Write element to History
	variable pos = numpnts(History)
	InsertPoints (pos), 2, History
	
	string LineRefElements ="LINEREFELEMENTS:"
	string LineRefEnergies = "LINEREFENERGIES:"
	
	variable pnts = numpnts(Lineref)
	for(i=0; i<pnts;i+=1)
		LineRefElements += GetDimLabel(LineRef, 0, i ) +";"
		LineRefEnergies += num2str(LineRef[i])+";"
	endfor
	History[pos]= LineRefElements +"*"
	History[pos+1]= LineRefEnergies +"*"
	
	string cmd = ""
	sprintf cmd, "CMD:iS_AnalyseCube(:cube, :packages:ispectra:LineRef, %g)", avgChannels
	iS_WriteString2History(cmd)
	
	Printf "Time required for extracting element intentisies: %f seconds\r", (stopMStimer(timeref)/1e6)
end


 //------------------------- click on to show spectrum---------------------

function iS_DisplayGreyScale(w)
	wave w
	
	dfRef dfr =  root:Packages:iSpectra:	
	
	DoWindow ImageZoom
	if (V_flag ==0)
		NewImage/K=1/N=ImageZoom  w
		SetWindow ImageZoom,hook(s)=iS_Hook	
	endif
end



Function iS_Hook(s)
	STRUCT WMWinHookStruct &s
	SVAR win = root:Packages:iSpectra:ActiveWindow
	
	wave/Z cube = root:cube
	wave/Z M_BaseImage = root:Packages:iSpectra:M_BaseImage
	
	if(!WaveExists(cube) || !WaveExists(M_BaseImage))
		return 0
	endif
	
	// store where the mouse is currently
	variable xpos =  AxisValFromPixel("", "Bottom", s.mouseLoc.h)
	variable ypos = AxisValFromPixel("", "Left", s.mouseLoc.v)
	
	variable xx = round (xpos / DimDelta(M_BaseImage, 0))
	variable yy =  round (ypos / DimDelta(M_BaseImage, 0))
	
	variable maxX = DimSize(cube, 0)
	variable maxY = DimSize(cube, 1)
	
	
	switch(s.eventCode)
		
		case 3: 				// handle left mouse click 	
		
			// prevent error when clicking outside of image
			if (xx >= maxX || yy >= maxY || xx < 0  || yy < 0)
				break
			endif
				
			MatrixOp/O root:Packages:iSpectra:CrsSpec = beam(cube, xx, yy)	
			wave CrsSpec = 	root:Packages:iSpectra:CrsSpec
			
//			// if background wave is present, add backround on again
//			if (WaveExists(root:bkg))
//				wave bkg		
//				CrsSpec += bkg
//			endif
			
			Setscale/P x, 0, (xPerChannel), "keV", CrsSpec		
			doWindow/F PointSpectrum
			
			if (V_flag==0)
				Display/K=1/N=PointSpectrum CrsSpec	
				SetAxis/W=PointSpectrum Bottom 0, (MaxkeVSpecVal)	
				Label/W=PointSpectrum left "Counts"
				ModifyGraph/W=PointSpectrum minor(bottom)=1
			endif
			
			//DoWindow/F $win
			break
						
	endswitch
	
	return 1
End




// *************************************************************************************************
// --------------------------------- Model Setup functions  -------------------------------------------------
// *************************************************************************************************


function iS_ModelPanel()
	
	dfRef dfr =  root:Packages:iSpectra:	
	wave Settings = dfr:Settings
	
	DoWindow/F ModelSetup
	if (V_flag)
		//DoWinDow/F ModelSetup
		return 1
	endif
		
	
	NewPanel /K=1/W=(885,199,1208,819)  /N = ModelSetup
	SetDrawLayer UserBack
	DrawRRect 18,170,300,305
	DrawText 38,195,"Model Setup"
	DrawRRect 18,320,300,455
	DrawText 38,350,"Compare and Merge Groups"
	DrawRRect 18,470,303,595
	DrawText 38,495,"Plot Intensities"
	
	DrawRRect 18,12,300,155
	DrawText 38,35,"Display Options"
	DrawText 38,100,"Generate RGB Map Image"
	
	
	
	SetVariable Start_keV,pos={56,209},size={104,15},bodyWidth=51,proc=iS_DrawLevels,title="start (keV)"
	SetVariable Start_keV,limits={0,15,0.05},value= root:Packages:iSpectra:Settings[%start]
	SetVariable thresh,pos={22,232},size={138,15},bodyWidth=51,proc=iS_DrawLevels,title="Threshold (counts)"
	SetVariable thresh,limits={0,200,1},value= root:Packages:iSpectra:Settings[%thresh]
	SetVariable MinPix,pos={62,257},size={98,15},bodyWidth=51,title="min pixels"
	SetVariable MinPix,value= root:Packages:iSpectra:Settings[%MinPix]
	CheckBox Erosion,pos={111,284},size={43,15},proc=iS_ErodeCheckBox,title="Erode"
	CheckBox Erosion,value= 0
	PopupMenu ErosionColor,pos={223,282},size={50,20},proc=iS_Color
	PopupMenu ErosionColor,mode=1,popColor= (100,100,100),value= #"\"*COLORPOP*\""
	PopupMenu ErosionMode,pos={163,282},size={52,20},proc=iS_ErosionSetMode
	PopupMenu ErosionMode,mode=1,popvalue="3x3",value= #"\"3x3;5x5\""
	Button DoIt,pos={234,228},size={50,20},proc=iS_runModel,title="Do it"
	SetVariable setCorrMatrixFactor,pos={75,364},size={140,15},title="Correlation factor"
	SetVariable setCorrMatrixFactor,limits={0,1,0.005},value= root:Packages:iSpectra:Settings[%vCorr]
	SetVariable ShowMatchesSetGroup,pos={38,389},size={177,15},title="Show Matching Groups for"
	SetVariable ShowMatchesSetGroup,limits={0,inf,1},value= _NUM:0
	Button CrossCorrDoIt,pos={223,360},size={50,20},proc=iS_CrossCorrelation,title="Do it"
	Button ShowMatchesDoIt,pos={223,385},size={50,20},proc=iS_CrossCorrelation,title="Do it"
	Button MergeSelection,pos={83,420},size={170,20},proc=iS_MergeSelection,title="Merge selected groups"
	PopupMenu PlotX,pos={39,505},size={60,20},title="Plot "
	PopupMenu PlotX,mode=1,popvalue="C",value= #"iS_ReturnElements()"
	PopupMenu PlotY,pos={129,505},size={53,20},title=" vs"
	PopupMenu PlotY,mode=1,popvalue="C",value= #"iS_ReturnElements()"
	Button PlotInt,pos={234,505},size={50,20},proc=iS_PlotHist,title="Do it"
	Button StartROI,pos={43,535},size={70,20},proc=iS_MyRoiDrawButtonProc,title="Start ROI"
	Button clearROI,pos={123,535},size={80,20},proc=iS_MyRoiDrawButtonProc,title="Delete ROI"
	Button GetPixels,pos={218,535},size={70,20},proc=iS_MySaveRoiCopyProc,title="Get Pixels"
	Button MakeGroup,pos={118,565},size={90,20},proc=iS_MakeGroupFromIntensitiesCtrl,title="Make Group"

	PopupMenu ImageSelector,pos={37,45},size={165,20},title="Base Image"
	PopupMenu ImageSelector,mode=1,popvalue="M_SumImage",value= #"\"M_SumImage;M_TIFImage;M_RGBImage;\"", proc = iS_BaseImageSelectorProc
	
	PopupMenu red,pos={20,110},size={80,20},bodyWidth=60,proc=iS_GenerateRGBMap,title="red"
	PopupMenu red,mode=1,popvalue="red",value= #"iS_ReturnElements()"
	PopupMenu green,pos={110,110},size={80,20},bodyWidth=60,proc=iS_GenerateRGBMap,title="green"
	PopupMenu green,mode=1,popvalue="green",value= #"iS_ReturnElements()"
	PopupMenu blue,pos={200,110},size={80,20},bodyWidth=60,proc=iS_GenerateRGBMap,title="blue  "
	PopupMenu blue,mode=1,popvalue="blue",value= #"iS_ReturnElements()"
EndMacro




Function iS_DrawLevels(ctrlName,varNum,varStr,varName) : SetVariableControl		// function to display energy and count cutoff on pixel spectrum
	String ctrlName
	Variable varNum
	String varStr
	String varName
	
	doWindow/F PointSpectrum
	if (V_flag==0)
			return 0	
	endif
	
	strswitch (ctrlname)
	
		case "Start_keV": // set vertical line		
	
			Getaxis/Q left
		
			SetDrawLayer/K /W=PointSpectrum UserFront   // Kill existing draw layer
			SetDrawLayer /W=PointSpectrum UserFront	// set new draw layer
			SetDrawEnv /W=PointSpectrum xcoord = bottom, ycoord = left, dash = 3, linefgc = (0, 0, 65535), linethick=0.6
			DrawLine varNum, V_min, varnum, V_Max
		break
	
		case "thresh":	// set horizontal line	
	
			Getaxis/Q bottom
		
			SetDrawLayer/K /W=PointSpectrum UserBack   // Kill existing draw layer
			SetDrawLayer /W=PointSpectrum UserBack	// set new draw layer
			SetDrawEnv /W=PointSpectrum xcoord = bottom, ycoord = left, dash = 3, linefgc = (0, 0, 65535), linethick=0.6
			DrawLine  V_min, varnum, V_max, varnum
			
		break
			
	endswitch
	
	doWindow/F ModelSetup
	return 1
End



Function iS_ErosionSetMode(ctrlName,popNum,popStr) : PopupMenuControl				//Set Erosion mode value in Settings
	String ctrlName
	Variable popNum
	String popStr
	
	dfRef dfr =  root:Packages:iSpectra:	
	wave/Z settings = dfr:Settings
	
	
	if (!WaveExists(settings))
		return 0
	endif
	
	
	StrSwitch(popstr)
		
		case "3x3":
			Settings[%ErosionMode] = 6      // set structure element parameter for ImageMorphology
			break
			
		case "5x5":
			Settings[%ErosionMode] = 5
			break	
	endswitch
	
	// if Erode is checked, run erosion with updated mode again
	
	Wave/Z M_GroupsErodeUndo = root:M_GroupsErodeUndo
	if(WaveExists(M_GroupsErodeUndo))
		iS_Erode(0)
		iS_Erode(1)
	endif
	
	return 1
End



Function iS_Color(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
		
	// write RGB values to Settings wave		
	dfRef dfr =  root:Packages:iSpectra:	
	wave/Z settings = dfr:Settings
	
	if (!WaveExists(settings))
		return 0
	endif
	
	
	Settings[%R] = str2num(popstr[1,inf])
	variable spos= strsearch(popstr,",",0)
	Settings[%G] = str2num(popstr[spos+1,inf])
	spos= strsearch(popstr,",",spos+1)
	Settings[%B]= str2num(popstr[spos+1,inf])
	
	
	// check if M_Groups Exists, if yes check if group 301 exists and change color
	wave/z M_Groups = root:M_Groups	
	if (!waveexists(M_Groups))
		return 0
	endif
	
	DoWinDow/F BaseImage
	ModifyImage M_Groups eval={301,Settings[%R],Settings[%G],Settings[%B]}
	DoWinDow/F ModelSetup
		
	return 1
End





// *************************************************************************************************
// --------------------------------- Merging  functions  -------------------------------------------------
// *************************************************************************************************


function iS_Merge(Target, Add2Target)
	// element lists as input for groups
	string Target, Add2Target
	

	
	dfRef dfr =  root:Packages:iSpectra:	
	wave/Z/T M_Model =dfr:M_Model
	wave/Z/T History =dfr:History
	Wave/Z M_Groups
	
	if (!WaveExists(M_Model) || !WaveExists(M_Groups))
		return 0
	endif
	
	// now find the group numbers of the corresponding strings; first make list of elements in groups
	variable i
	string List=""
	for(i=0;i<Dimsize(M_Model,0); i+=1)
		List +=  M_Model[i][%List] +";"	
	endfor	
	
	// Find Target Group
	variable TargetGroup = WhichListItem(Target, List, ";")
	
	// Find group to be merged; use start index to start search AFTER TargetGroup; this prevents errors if elemental contents are identical
	variable Add2TargetGroup = WhichListItem(Add2Target, List, ";", TargetGroup+1)

	
	if (TargetGroup==-1)
		printf "WARNING: Target group does not exist: %s ;  ", Target
		printf "Could not merge %s with %s \r", Add2Target, Target
		return -1
	endif
	
	
	if (Add2TargetGroup==-1)
		printf "WARNING: Group does not exist: %s ;  ", Add2Target
		printf "Could not merge %s with %s \r", Add2Target, Target
		
		//if Add2Target does not exists, still add elements to target list, this reduces failed pixel assigments in batch processing
		if(TargetGroup!=0) 
			M_Model[TargetGroup][%List] += Add2Target
			M_Model[TargetGroup][%List] = iS_CollapseDuplicatesInList(M_Model[TargetGroup][%List], ",", 1)
		endif
		
		return -1
	endif
	
	// Add pixels and element string to target
	MultiThread M_groups = M_Groups == Add2TargetGroup ? TargetGroup : M_groups
	
	//update The Elements List of TargetGroup, but  don't overwrite TRASH
	if(TargetGroup!=0) 
		M_Model[TargetGroup][%List] += M_Model[Add2TargetGroup][%List]
		M_Model[TargetGroup][%List] = iS_CollapseDuplicatesInList(M_Model[TargetGroup][%List], ",", 1)
	endif
	
	// if Add2Target has a name but target doesn't --> use name
	if (strlen(M_Model[TargetGroup][%Phase]) == 0 && strlen(M_Model[Add2TargetGroup][%Phase]) != 0)
		M_Model[TargetGroup][%Phase] = M_Model[Add2TargetGroup][%Phase]
	endif
	
	// Update pixel counts 
	variable TargetGroup_pix = str2num(M_Model[TargetGroup][%Pixels])
	variable  Add2TargetGroup_pix = str2num(M_Model[Add2TargetGroup][%Pixels])
	M_Model[TargetGroup][%Pixels] = num2Str(TargetGroup_pix + Add2TargetGroup_pix)
	
	// Delete Add2Target group and collapse Model wave
	DeletePoints/M=0 Add2TargetGroup, 1, M_Model
	M_Model[][%Group] = num2str(p)
	MultiThread M_Groups = M_Groups > Add2TargetGroup ? M_Groups-1 : M_Groups 	
	
	// Write to History
	string cmd = ""
	sprintf cmd, "CMD:iS_Merge(\"%s\",\"%s\")", Target, Add2Target
	iS_WriteString2History(cmd)
	
	return TargetGroup
end 


Function iS_MergeSelection(ctrlName) : ButtonControl
	string ctrlname
	
	dfRef dfr =  root:Packages:iSpectra:	
	wave/Z/T M_Model =dfr:M_Model
	wave/Z/T History = dfr:History
	Wave/Z M_Groups = root:M_Groups
	
	if (!WaveExists(M_Model) || !WaveExists(M_Groups))
		return 0
	endif
	
	Make/FREE/T/O/N=0 tmp
	
	variable i, j
	
	// get element content of each group which is active store as text wave
	for(i=0; i< DimSize(M_Model, 0); i+=1)		
		if (str2num(M_Model[i][%Show]) == 1)
			InsertPoints j, 1, tmp
			tmp[j] = M_Model[i][%List]
			j+=1
		endif			
	endfor
		
	String Target = tmp[0]
	
	// start at the last entry of tmp and add this group to the target
	for(i=DimSize(tmp, 0) - 1; i> 0; i-=1)	
		
		// now merge groups; the return value is needed to update the panel at the end of the function
		variable TargetGroup = iS_Merge(Target, tmp[i])
			
		// now update target name, as it changes once Add2Target is added unless target group == 0  (TRASH)
		if(TargetGroup!=0) 
			Target += tmp[i]
			Target= iS_CollapseDuplicatesInList(Target, ",", 1)		
		endif
	
	endfor
		
	//iSpectraGroupPanel() // re build group panel with new grous
	iS_UpdatePixelCount()
	iS_ClearPhases("checkAll")
	iS_ShowPhases(TargetGroup, 1)
	iS_iSpectraGroupPanel()
end




Function/S iS_CollapseDuplicatesInList(listStr, listSepStr, matchCase)
	String listStr, listSepStr
	Variable matchCase
	
	Variable i, n= ItemsInList(listStr, listSepStr)
	Make/FREE/T/FREE/N=0 T_dest
	Make/FREE/N=(n)/O/T T_list = StringFromList(p, listStr, listSepStr) // convert separated list to text wave
	Make/FREE/N=(n)/O W_idx, W_flag, W_orig=p
	
	// second sort key ensures that the duplicates stay in their original order
	if (matchCase)
		MakeIndex/C {T_list,W_orig} W_idx
	else
		MakeIndex {T_list,W_orig} W_idx
	endif
 
	W_flag = p==0 ? 0 : !cmpstr(T_list[W_idx[p]], T_list[W_idx[p-1]], matchCase)
	// this one-liner does the same as the for loop
	 W_orig = W_flag ? W_orig[p-1] : W_idx[p]

	Sort W_idx W_orig, W_flag 	// O(N.logN) solution to the O(N) problem of inverting the permutation that sorted the list

	Extract/T T_list, T_dest, ! W_flag
 
	Grep/E=".*"/Q/LIST=listSepStr T_dest // convert text wave to separated list
	Return S_Value
End






// *************************************************************************************************
// ---------------------------------  Build Groups Panel  ----------------------------------------------------
// *************************************************************************************************

Function iS_iSpectraGroupPanel()
	
	dfRef dfr =  root:Packages:iSpectra:	
	wave/Z/T model= dfr:M_Model
	
	if (!WaveExists(model))
		return 0
	endif
	
	variable i, n
	variable nGroups = DimSize(model, 0)
	
	variable top, left, width  	// sets basic size parameters for panel; to be set later
	variable space = 22  		//vertical space between group rows
	
	//Get the position of the panel if already open
	DoWindow iSpectraGroups
	if (V_flag == 1)    //Panel exists --> //Recreate panel at its last position
		GetWindow iSpectraGroups wsizeRM	
		top = V_top*ScreenResolution/72
		left = V_left * ScreenResolution/72				
		DoWindow /K iSpectraGroups
		NewPanel /N=iSpectraGroups /K=1 /W=(left, top, (left+600) , (top+((space)*nGroups)+70) ) 
		ModifyPanel/W=iSpectraGroups noEdit=0//, fixedSize=1  // disable modification on its size
	else  //Panel does not exists --> create at default position
		left = 200
		top = 400
		NewPanel /N=iSpectraGroups /K=1 /W=(left,top,(left+600),(400+(space*nGroups)+70) )
	endif
		
	variable xx = 40	// default x and y spacings
	variable yy = 55
	

	DrawText 10, 30, "Group"
	DrawText xx+10, 30, "Color"
	DrawText xx+60, 30, "Phase"
	DrawText  xx+160, 30, "Elements"
	DrawText  xx+260, 30, "Show"
	DrawText  xx+300, 30, "Pixels"
	DrawText  xx + 360, 30, "Pix%"
	DrawText  xx+420, 30, "EDS Pixels"
	
	Button $"checkAll", pos={xx+260, 35},size={30,15}, fsize = 8, title="Clear", proc = iS_ClearPhases
	Button $"allSpectra", pos={530, 30},size={40,20}, title="All", proc = iS_getSpectrumCtrl
	
 	for (i=0; i<(nGroups); i+=1)
 		
 		// set background color according to phase abundance
 		variable red = 65535, blue = 65535, green = 65535   //this would be white
 		if (str2num(Model[i][%PixPercent]) < MinPhaseFraction)
 			red = 65535
 			blue = 0
 			green = 0
 		endif
 			
 		SetVariable $"Group" + num2str(i), title = " ", pos = {20, yy+(i*space)}, size= {20, 50}, noedit = 1, win = iSpectraGroups, value = Model[i][%Group]	
 		PopupMenu $("Color" + num2str(i)), pos = {xx+5, yy+(i*space)}, size= {18, 50}, win = iSpectraGroups,	popcolor = (str2num(Model[i][%R]) , str2num(Model[i][%G]),str2num(Model[i][%B])), value="*COLORPOP*", proc = iS_ColorPopMenuProc
		SetVariable $"Phase" + num2str(i), title = " ", pos = {xx+60, yy+(i*space)}, size= {90, 50}, win = iSpectraGroups, value = Model[i][%Phase], proc =  iS_LabelWave
		SetVariable $"List" + num2str(i), title = " ", pos = {xx+160, yy+(i*space)}, size= {100, 50}, win = iSpectraGroups, value = Model[i][%List]
		CheckBox $"Show" + num2str(i), title = " ", pos = {xx+270, yy+(i*space)}, size= {40, 50}, value= str2num(Model[i][%Show]), win =iSpectraGroups, proc = iS_ShowPhasesCtrl
		SetVariable $"Pixels" + num2str(i), title = " ", pos = {xx+300, yy+(i*space)}, size= {50, 50}, noedit = 1, win = iSpectraGroups, value = Model[i][%Pixels]
		SetVariable $"PixPerc" + num2str(i), title= " ", pos =  {xx+360, yy+(i*space)}, size= {50, 50}, noedit = 1, win = iSpectraGroups, value = Model[i][%PixPercent], valueBackColor=(red, blue,  green )
		SetVariable $"EDSPixels" + num2str(i), title = " ", pos = {xx+420,yy+(i*space)}, size= {50, 50}, noedit = 1, win = iSpectraGroups, value = Model[i][%EDSPixels]
		Button $"EDS"+ num2str(i), pos={530, yy+(i*space)},size={40,20}, title="EDS", proc = iS_getSpectrumCtrl
		
	endfor
End




// *************************************************************************************************
// ---------------------------------  Panel functions ----------------------------------------------------
// *************************************************************************************************


function iS_ClearPhases(ctrlname)
	String ctrlname
	
	dfRef dfr =  root:Packages:iSpectra:	
	wave/T model= dfr:M_Model
	variable i
	
	for (i=0; i<DimSize(model, 0); i+=1)
		iS_ShowPhases(i, 0)
	endfor
		
	iS_iSpectraGroupPanel() // re-create panel with unchecked boxes
	return 0
end


Function iS_ShowPhasesCtrl(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	variable group; 	sscanf ctrlname, "Show%g", group 
	
	iS_ShowPhases(group, checked)
	DoWindow/F iSpectraGroups
	return 1
End


Function iS_ShowPhases(group, check)
	variable group
	Variable check
	
	dfRef dfr =  root:Packages:iSpectra:	
	wave/T model= dfr:M_Model
	wave M_Groups	
	
	DoWindow BaseImage
	if (V_flag==0)
		 iS_OpenGroupsOnGrey()
	endif
	//DoWindow/F Groups 
	DoWindow/F BaseImage
	
	if (check == 1)
		Model[group][%Show] =num2str(1) 
		ModifyImage M_Groups eval={group, str2num(Model[group][%R]), str2num(Model[group][%G]), str2num(Model[group][%B])}	
	else  
		Model[group][%Show] =num2str(0) 			
		ModifyImage M_Groups eval={group,-1,-1,-1}		// make pixels of that group transparent
	endif
	//DoWindow/F iSpectraGroups
End


Function iS_LabelWave(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum	
	String varStr		
	String varName		
	
	varStr = CleanUpName(VarStr,0)
		
	dfRef dfr =  root:Packages:iSpectra:	
	wave/T model= dfr:M_Model
	variable group; 	sscanf ctrlname, "Phase%g", group  
	
	Model[group][%Phase]	= varStr 
	SetDimLabel 0, group, $varStr, Model
End


Function iS_ColorPopMenuProc(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	variable r,g,b 
		
	iS_MyRGBstrToRGB(ctrlName, popStr,r,g,b)		
	return 1
End

Function iS_MyRGBstrToRGB(ctrlName, rgbStr,r,g,b)
	String ctrlName, rgbStr
	Variable &r, &g, &b
	
	wave M_Groups
	
	dfRef dfr =  root:Packages:iSpectra:	
	wave/T model= dfr:M_Model
	variable group; 	sscanf ctrlname, "Color%g", group  
	
	r= str2num(rgbStr[1,inf])
	variable spos= strsearch(rgbStr,",",0)
	g= str2num(rgbStr[spos+1,inf])
	spos= strsearch(rgbStr,",",spos+1)
	b= str2num(rgbStr[spos+1,inf])
	
	Model[group][%R] = num2str(r)
	Model[group][%G] = num2str(g)
	Model[group][%B] = num2str(b)
	
	DoWindow/F BaseImage
	if (!V_Flag)
		return 0
	endif
	
	if(str2num(model[group][%Show]))
		ModifyImage M_Groups eval={group, str2num(Model[group][%R]), str2num(Model[group][%G]), str2num(Model[group][%B])}
	endif	
	DoWindow/F iSpectraGroups
	
	return 1
End

// *************************************************************************************************
// ---------------------------------  Run Model and Group -------------------------------------------------
// *************************************************************************************************

Function iS_runModel(ctrlName) : ButtonControl
	String ctrlName

	dfRef dfr =  root:Packages:iSpectra:	
	wave/Z settings = dfr:Settings
	wave/Z cube = root:cube
	wave/z M_BaseImage = dfr:M_BaseImage	
	
	if (!WaveExists(cube) || !WaveExists(settings)  || !WaveExists(M_BaseImage))
		return 0
	endif
	
	variable returnVal = iS_FindGroups(root:cube, Settings[%thresh], Settings[%start], Settings[%MinPix] )
	
	// if FindGroup() aborted, quit here
	if (returnVal < 0)
		return -1
	endif
	
	
	// M_Groups and M_Model are created in FindGroups()
	Wave/Z M_Groups = root:M_Groups
	wave/Z/T M_Model = dfr:M_Model
	if (!WaveExists(M_Groups) || !WaveExists(M_Model))
		return 0
	endif
	
	
	iS_DisplayGroupsOnGrey(M_Model, M_Groups, M_BaseImage)
	iS_UpdatePixelCount()
	iS_iSpectraGroupPanel()	
		
	DoWindow/F iSpectra
End



function iS_FindGroups(cube, thres, start, minPix)							
	wave cube
	variable thres, start, minPix
	
	dfRef dfr =  root:Packages:iSpectra:	
	wave/Z LineRef = dfr:LineRef
	wave/Z Settings = dfr:Settings
	wave/T/Z History = dfr:History
	
	if (!DataFolderExists("Intensities"))
		DoAlert 0,"No Folder Intensities: run ...AnalyseCube(Cube)... first"
		return -1
	endif
	
	if (!WaveExists(LineRef))
		DoAlert 0, "No LineRef.ibw"
		return -2
	endif
	
		
	variable i
	variable ref1= startMStimer

	// Make masks for each element with 1 for Intensity being above threshold and 0 for below	
	String dfsave = GetDataFolder(1)
	
	SetDataFolder Intensities
	string ListOfElements= WaveList("*",";","")
	
	NewDataFolder/O root:Masks
	
	String Element=""
	for (i=0; i<(ItemsInList(ListOfElements)); i+=1)
		
		Element = StringFromList(i, ListOfElements)
		wave w = $Element
		
		Duplicate/O w $"root:Masks:"+Element
		wave M_Mask = $"root:Masks:"+Element
		Multithread M_Mask = M_Mask > thres ? 1 : 0
	endfor
		
	SetDataFolder dfsave

	//find 1st element above the start energy level
	SetDataFolder Masks							
			
	FindLevel/Q LineRef start			
	variable low = 0
	if (V_flag == 0)
		low = ceil(V_LevelX)
	endif
	
	// Make destination wave from masks with elements above energy start threshold and make mask-cube from it
	Make/FREE/D/O/N=(DimSize(cube,0), DimSize(cube, 1)) Result=0
	
		
	// List of elements present above start kV energy range  but below max voltage devided by overvoltage
	String ElementList =""
	variable nChannels = DimSize(cube, 2)
	variable maxVoltage = Settings[%AccV]
	
	// introduce an overvoltage factor 
	variable maxEnergy = maxVoltage / OverVoltage
	FindLevel/Q LineRef, maxEnergy
	variable MaxElement = numpnts(LineRef)
	// if level was found, what is max element
	if(V_flag==0)
		MaxElement =trunc(V_LevelX)	+1
	endif

	for (i=low; i<MaxElement; i+=1)
		ElementList += ( GetDimLabel(LineRef, 0, i) + ";" )
	endfor
	
//	ElementList=RemoveEnding(ElementList, ";")
	Concatenate/O ElementList, dest  
	
	for (i=0; i<DimSize(dest,2); i+=1)
		MultiThread Result += dest[p][q][i] * 2^i			//create integer number wave
	endfor
	
	KillWaves/Z dest
	SetDataFolder dfsave
		
	//Collapse sorted wave to single unique values
	
	Make/FREE/O/N=(DimSize(cube,0) * DimSize(cube, 1)) Integer1D
	Make/FREE/O/N=0 tmp
	MultiThread Integer1D = Result[p]
	sort Integer1D Integer1D
	

	ImageStats Result //Get Minimum and Maximum values
		
	variable num, Level
	level = V_min
	
	i=0
	do
		InsertPoints i, 1, tmp
		tmp[i] = level
		FindLevel/Q/P /R=(num, numpnts( Integer1D))  Integer1D, level+1
		if (numtype(V_levelX) == 2)
			break
		endif
		num=ceil(V_levelX)
		level=Integer1D[num]
		i+=1	
				
	while(1)
	
	
	Make/FREE/T/O/N=(numpnts(tmp)) Elements, ListOfGroups=""	  // convert Integers into binary strings
		
	// convert binary sequence into element combinations
	variable n, k
	string buffer
	variable length=numpnts(LineRef) - low
	
	
	String sGroupList=""
	String formatstr								
	formatstr = "%0"+num2str(length) + "b"	// nessecary to convert integer into binary
	
	for (i=0; i<DimSize(tmp,0); i+=1)				
		
		sprintf buffer formatstr, tmp[i]			// convert Integer into binary using sprintf
		 elements[i] = buffer
		 
		k=0 +low
		n=length-1
		do
		 	
		 		string char = buffer[n]	 		
		 		if (cmpstr(char, "1") == 0)		// if 1 get element name
		 			ListOfGroups[i] += GetDimLabel(LineRef, 0, (k)) +","
		 		endif
		 		k+=1
		 		n-=1
		 		
		 		
		 	while(n>-1)
		 	
		// if a group does not contain any element, call it empty	
		if (cmpstr(ListOfGroups[i], "")==0)
			ListOfGroups[i] = "EMPTY,"
		endif 
		
		sGroupList += ListOfGroups[i]+";"	
		 
	endfor
		
	// set a group TRASH on top of the list 
	InsertPoints 0, 1, ListOfGroups, tmp
	ListOfGroups[0] = "TRASH,"
	tmp[0] = 0
	
		
	// Make wave M_Groups with group number from 0 to n
	Make/O/W/N=(DimSize(cube,0), DimSize(cube, 1)) M_Groups = 0
	iS_ScaleIfMagIsKnown(M_Groups)
	
	for (i=0; i<DimSize(ListOfGroups,0); i+=1)	
		MultiThread M_Groups = Result == tmp[i] ? i : M_groups	
	endfor
	
	
	// Set up M_Model - the model parameter wave - from M_Groups
	
	variable numPixels = DimSize(M_groups, 0) * DimSize(M_groups, 1)
	Duplicate/O/FREE M_groups, M_tmp
	string labels = "Group;R;G;B;List;Pixels;PixPercent;EDSPixels;Phase;Show;"
	Make/O/T/N=(numpnts(ListOfGroups), ItemsInList(labels)) root:packages:ispectra:M_Model =""
	wave/T M_model =  root:packages:ispectra:M_Model
	
	iS_WaveLabel(M_Model, labels, 1)			//label model wave and fill in some values
	M_Model[][%List] = ListOfGroups[p]
	M_Model[][%Group] = num2str(p)
	M_Model[][%Show] = num2str(1)
	
	
	// set colors of groups 
	Colortab2wave rainbow
	wave M_colors
	variable randomColor
	
	for (i=0; i<(numpnts(ListOfGroups)); i+=1)
		
		// set EMPTY or TRASH to black (0,0,0)	
		if (cmpstr(ListOfGroups[i], "EMPTY,") == 0  || cmpstr(ListOfGroups[i], "TRASH,") == 0)
			M_Model[i][%R] = num2str(0), M_Model[i][%G] = num2str(0), M_Model[i][%B] = num2str(0)
		else
		
			randomColor = trunc(abs(enoise(99)))	  //get random row from M_Colors
				
			M_Model[i][%R] = num2str(M_colors[randomColor][0])
			M_Model[i][%G] =  num2str(M_colors[randomColor][1] )
			M_Model[i][%B] =  num2str(M_colors[randomColor][2] )
		endif
		
		MultiThread M_tmp = M_Groups == i ? 1 : 0
		MatrixOP/FREE Summe =sum(M_tmp)
		M_Model[i][%Pixels] = num2str(Summe[0])
	endfor
	
	// merge all groups with less than minPixel with TRASH
	for (i=1; i<DimSize(M_Model,0); i+=1)
		
		if (str2num(M_Model[i][%Pixels]) <= minPix)
			
			MultiThread M_groups = M_Groups == i ? 0 : M_groups		// merges groups with less than MinPixels with TRASH 
			
			variable  PixelsOfTrashedPhase = str2num(M_Model[i][%Pixels])
			M_Model[0][%Pixels]  += num2Str( PixelsOfTrashedPhase)  // update pixel count of trash
	
			DeletePoints/M=0 i, 1, M_Model
	
			M_Model[][%Group] = num2str(p)
			MultiThread M_Groups = M_Groups > i ? M_Groups-1 : M_Groups 
			i-=1
		endif
	endfor
	
	KillWaves/Z M_Colors
	KillDataFolder/Z Masks	
	
	
	variable nGroups = Dimsize(M_Model, 0)
	if (nGroups>MaxPhases)
			String msg =  "More than " +num2str(MaxPhases)+ " groups detected! Change model parameters!"   
			DoAlert 0, msg
			return -1
	endif
	

	//write to History	
	String cmd = ""
	sprintf cmd, "CMD:iS_FindGroups(root:cube, %g, %g, %g)", thres,start,MinPix
	iS_WriteString2History(cmd)
	
	printf "Time required to run model: %f seconds\r", (stopMStimer(ref1)/1e6)
	printf " Number of Groups found: %g \r", nGroups
	return 1
end



function iS_WaveLabel(w, str, dim)
	wave w
	string str
	variable dim		// 0 for rows, 1 for colums
	
	if (ItemsInList(str) != DimSize(w, dim))
		print NameOfWave(w)
		print "No labeling possible"
		return -1
	endif
	
	variable i = 0
	for(i=0; i<dimSize(w, dim); i+=1)
		SetDimLabel	dim, i, $StringfromList(i, str), w
	endfor
	
	return 1
end



function iS_getSpectrumCtrl(ctrlname)
	String ctrlname
	
	if (cmpstr(ctrlname, "allSpectra") == 0)						// when "All" is pressed
		wave/T M_Model = root:packages:ispectra:M_Model
		variable i
		for (i=0; i<DimSize(M_Model, 0); i+=1)
			iS_getSpectrum(i)
		endfor	
		return 1
	endif
		
	variable group; 	sscanf ctrlname, "EDS%g", group  		// any of the EDS buttons pressed
	iS_getSpectrum(group)	
	return 0
end


function iS_getSpectrum(group)
	variable group
	
	dfRef dfr =  root:Packages:iSpectra:	
	wave settings =dfr:Settings
	wave/T M_Model = dfr:M_Model
	wave M_Groups = root:M_Groups
	wave cube = root:cube
	
			
	string dfsave = GetDataFolder(1)
	
	If (DataFolderExists("Spectra") == 0)		//Does DataFolder exist already?
		NewDatafolder Spectra
	endif
	
	SetDataFolder Spectra	
	
	// use either given name or group name for spectrum
	string wName = ""
	if (strlen(M_Model[group][%Phase]) == 0)  // group has no name
		wName = "EDS_Group_" + num2str(group)
	else
		wName = M_Model[group][%Phase]
		//wName = CleanUpName(wName, 0)
	endif	
		
		
	Make/O/N=(DimSize(cube, 2)) $wName
	wave EDS = $wName
	
	Duplicate/O/FREE M_Groups, Mask   //create mask from M_Groups
	MultiThread Mask = M_Groups == Group ? 1 : 0
	MultiThread EDS = iS_WorkerFunc(cube, Mask, p)	
	SetScale/P x, 0, (xPerChannel), "keV" EDS
	
	// add a wave note with number of pixels
	variable nPix = sum(Mask)
	Note EDS, "MODE:full;PIXELS:" + num2str(nPix)

	SetDataFolder dfsave
	
	// display spectrum if not done already
	// is a spectrum graph out there?
	iS_SetSpectrumGraph("SpectrumDisplay")
	iS_DisplayWave(wName, "SpectrumDisplay", "AddSpectrum") 	
end





ThreadSafe Function iS_WorkerFunc(cube, mask, plane)
	wave cube, mask
	variable plane
 
	MatrixOP /FREE W_Sum = sum(mask * cube[][][plane])
	return W_Sum[0]
End
 


 

function iS_UpdatePixelCount()			// updates pixelcounts in M_Model wave
	wave/Z M_Groups = root:M_groups
	wave/Z/T M_Model = root:packages:ispectra:M_Model
	
	if (!WaveExists(M_Groups) || !WaveExists(M_Model))
		return 0
	endif
	
	// if M_Groups is eroded, restore original values
	wave/Z M_GroupsErodeUndo= root:M_GroupsErodeUndo
	if(WaveExists(M_GroupsErodeUndo))
		duplicate/FREE M_Groups buffer
		M_Groups=M_GroupsErodeUndo
	endif
		
	variable pixtotal = DimSize(M_Groups, 0) * DimSize(M_Groups, 1)
	Duplicate/O/FREE M_Groups, M_tmp
	

	// determine pixel fraction
	variable i, tmp
	string stmp
	
	for(i=0; i<DimSize(M_Model, 0); i+=1)		
		MultiThread M_tmp = M_Groups == i ? 1 : 0
		MatrixOP/FREE Summe =sum(M_tmp)
		M_Model[i][%Pixels] = num2str(Summe[0])
	
		// calculate percentage
		tmp =  Summe[0] /pixtotal *100
		// set significant digit in display depending on value	
		if (tmp < 1)	
			sprintf stmp, "%.2f", tmp
		else	
			sprintf stmp, "%.1f", tmp
		endif
		
		M_Model[i][%PixPercent] = stmp	
	endfor
	
	// if M_Groups WAS eroded restore eroded values
	if(WaveExists(M_GroupsErodeUndo))
		M_Groups = buffer
	endif
	
end




// *******************************************


Function iS_ErodeCheckBox(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	
	wave/Z M_Groups = root:M_Groups
	dfRef dfr =  root:Packages:iSpectra:	
	wave/Z settings = dfr:Settings
	
	if (!WaveExists(M_Groups) || !WaveExists(Settings))
		return 0
	endif
		
//	Settings[%Erosion] = checked
	iS_Erode(checked)	
	
	DoWindow/F iSpectra	
End





function iS_Erode(mode)
	variable mode 		// mode is 1 for do erosion or 0 for undo
	
	dfRef dfr =  root:Packages:iSpectra:	
	wave/Z settings = dfr:Settings
	wave/Z/T M_Model = dfr:M_Model
	wave/Z M_Groups = root:M_Groups
	
	if (!WaveExists(M_Groups) || !WaveExists(Settings)  || !WaveExists(M_Model))
		return 0
	endif
	
	Settings[%Erosion] = mode
	
	// if mode = 0 
	if (mode==0)
		wave/Z M_GroupsErodeUndo = root:M_GroupsErodeUndo
		if(!WaveExists(M_GroupsErodeUndo))
			return 0
		endif
		
		// restore M_Groups
		M_Groups = M_GroupsErodeUndo
		KillWaves/Z M_GroupsErodeUndo
		
		return 0	
	endif
	
	// if mode != 0; do erosion
	wave/Z M_GroupsErodeUndo = root:M_GroupsErodeUndo
		if(WaveExists(M_GroupsErodeUndo))
			return -1
	endif
		
	Duplicate/O M_Groups, M_GroupsErodeUndo
	
	variable xx = DimSize(M_groups, 0)
	variable yy = DimSize(M_Groups, 1)
	
	Make/FREE/O/N=(xx,yy) Filtered =0
	
	NewDataFolder/O/S Masks
	
	variable i
	for (i=0; i<DimSize(M_Model,0); i+=1)
		Make/B/U/O/N=(xx,yy) $"Mask_Group_" + num2str(i)		
		wave w = $"Mask_Group_" + num2str(i)
		
		MultiThread w = M_Groups == i ? 255 : 0								// Make Mask from every group
		ImageMorphology/E=(Settings[%ErosionMode])/L BinaryErosion w		// erode it, use ErosionMode set using Popup
		Wave M_ImageMorph
		MultiThread w = M_ImageMorph == w ? 0 : 255				// which pixels have been eroded?
		MultiThread Filtered +=w									// add to filter wave, which contains all eroded pixels
	endfor				
	
	SetDataFolder root:
	KillDataFolder/Z Masks
	
	MultiThread M_Groups = Filtered == 255 ? 301 : M_Groups		// set all pixels which have been filtered to 301
	DoWinDow/F BaseImage
	
	if(V_flag)
		ModifyImage M_Groups eval={301,Settings[%R],Settings[%G],Settings[%B]}
	endif
	

	Make/FREE/O/N=(xx,yy) M_tmp					// count how many pixels are left in each phase
	M_Model[][%EDSPixels] = ""
	i= 0
	for (i=0; i<DimSize(M_Model,0); i+=1)
		MultiThread M_tmp = M_Groups == i ? 1 : 0
		MatrixOP/FREE Summe =sum(M_tmp)
		M_Model[i][%EDSPixels] = num2str(Summe[0])
	endfor		
	
	return 1	
end



// *************************************************************************************************
// --------------------------------- Intensity plot functions----------------------------------------------
// *************************************************************************************************


function iS_PlotHist(ctrlName) : ButtonControl		// executes when "Do it" in Plot Intensities section is hit
	String ctrlName
	
	dfRef dfr =  root:Packages:iSpectra	
	Wave/Z Settings = dfr:Settings
	SVAR/Z xElem = dfr:xElem
	SVAR/Z yElem = dfr:yElem
	SVAR/Z ActivePhases = dfr:ActivePhases
	
	ControlInfo /W=ModelSetup PlotX
	xElem = S_Value
	
	ControlInfo /W=ModelSetup PlotY
	yElem = S_Value
	
	
	wave/Z xx = $"Root:Intensities:"+xElem
	wave/Z yy = $"Root:Intensities:"+yElem
		
	if (!WaveExists(settings) || !WaveExists(xx)  || !WaveExists(yy) )
		return 0
	endif
	
	// get a string of all groups that are displayed
	ActivePhases = iS_DisplayedPhases()
	if (strlen(ActivePhases) == 0)
		return 0
	endif
	
	// make 2D histogram, function creates M_Hist and M_ActivePhases
	iS_MakeHistogram(xx, yy, activePhases)

	//Plot Histigram
	iS_PlotHistogram(xElem, yElem, root:IntPlotData:M_Hist)	
end



function/S iS_DisplayedPhases()
	dfRef dfr =  root:Packages:iSpectra:	
	wave/Z/T M_Model =dfr:M_Model
	
	String Groups=""
	
	if (!WaveExists(M_Model) )// || !WaveExists(M_Groups))
		return Groups
	endif
		
	variable i
	for(i=0; i<DimSize(M_Model, 0); i+=1)
		
		if(str2num(M_model[i][%Show]) == 0)		//if phase not shown, skip it
			continue
		endif	
		
		Groups += M_Model[i][%List] +";"
	endfor	
	return Groups	
end





function iS_MakeHistogram(xx, yy, activePhases) // xx and yy are 2D element maps
	wave/Z xx, yy
	string activePhases
	
	
	dfRef dfr =  root:Packages:iSpectra:	
	wave/Z/T M_Model =dfr:M_Model
	wave/Z M_Groups = root:M_Groups
	SVAR/Z ActPhases = dfr:ActivePhases
	
	//needed for History
	ActPhases = ActivePhases
	
	if (!WaveExists(M_Model) || !WaveExists(M_Groups))
		return M_Groups
	endif
	
	variable rows = DimSize(xx, 0)
	variable cols = DimSize(xx, 1)
		
	Duplicate/FREE/O xx, xtmp
	Duplicate/FREE/O yy, ytmp
	
	//raw data are ready, now create folder for the histogram
	string dfsave = GetDataFolder(1)
	NewDataFolder/O/S root:IntPlotData
	
	Make/B/U/O/N=(rows, cols) M_ActivePhases=0
	
	variable i
	for(i=0;  i<DimSize(M_Model, 0); i+=1)
		// get a mask of all pixels that are being displayed and active from the string of active phases	
		string ThisGroup = M_Model[i][%List]
		variable groupNum = WhichListItem(ThisGroup, activePhases, ";") 
		if (GroupNum != -1)
			MultiThread M_ActivePhases += M_Groups == i ? 1 : 0
		else
			// if groups are not included, e.g. during batch processing, though a warning 
		//	printf "WARNING: Phase  %s  does not exists and cannot be incuded in density plot!\r", ThisGroup 
		endif
	endfor	
	
	// check if there are any active phase at all
	if(sum(M_ActivePhases) == 0)
		print "No Pixels Selected! Cannot proceed with intensity plots"
		SetDataFolder DFsave
		return -1
	endif
	
	// exclude pixels in maps which are not shown
	MultiThread xtmp *= M_ActivePhases
	MultiThread ytmp *= M_ActivePhases
	
	// redimension into 1D and double
	Redimension/D/N=(rows * cols) xtmp, ytmp
	
	// exclude zeros
	MultiThread xtmp = xtmp == 0 ? NaN : xtmp
	MultiThread ytmp = ytmp == 0 ? NaN : ytmp
	
	//make histogram - based on WM bivariate histogram ipf
	variable xbins = floor(WaveMax(xtmp))
	variable ybins = floor(waveMax(ytmp))
		
	Make/O/N=(xbins, ybins) M_Hist = 0
	
	Variable pnts = numpnts(xtmp)
	if (pnts != numpnts(ytmp))
		return 0
	endif
	
	Variable  xpnt, ypnt, dx, dy, x0, y0
	dx = DimDelta(M_Hist, 0)
	dy = DimDelta(M_Hist, 1)
	x0 = DimOffset(M_Hist, 0)
	y0 = DimOffset(M_Hist, 1)
	
	Variable maxXpnt = dimSize(M_Hist, 0)
	Variable maxYpnt = dimSize(M_Hist, 1)

	for (i = 0; i < pnts; i += 1)
		if ( (numtype(xtmp[i]) != 0) || (numtype(ytmp[i]) != 0) )   // avoid NaN and Inf
			continue
		endif
		xpnt = floor((xtmp[i]-x0)/dx)
		ypnt = floor((ytmp[i]-y0)/dy)
		
		if ( (xpnt >= maxXpnt) || (ypnt >= maxYpnt) )
			continue
		endif
		M_Hist[xpnt][ypnt] += 1
	endfor
	
	SetDataFolder DFsave
	
	return 1
end

function iS_PlotHistogram(xElem, yElem, M_Hist)
	string xElem, yElem		// for annotations
	wave/Z M_Hist
	
	if (!WaveExists(M_Hist))
		return 0
	endif
	
	DoWindow/F biHist
	if(V_flag == 0)
		Display/K=1 /N=biHist /W=(389,308,1095,734)
		AppendImage/T M_Hist
		
		ModifyImage M_Hist ctab= {1,*,Terrain,0};DelayUpdate
		ModifyImage M_Hist minRGB=(0,0,0),maxRGB=0
		
		//ModifyImage M_Hist ctab= {*,*,Magenta,0}
		ModifyGraph margin(left)=60,margin(bottom)=40,margin(top)=80,margin(right)=99
		ModifyGraph mirror=2, nticks(left)=2,nticks(top)=4, minor = 1, fSize=9,  standoff=0
		ModifyGraph nticks(left)=2,nticks(top)=4	
		ModifyGraph tkLblRot(left)=90
		ModifyGraph btLen=3
		ModifyGraph tlOffset=-2
		SetAxis/A/R left
		Label left "\\Z16  counts " +yElem
		Label top "\\Z16  counts "+xElem
		ColorScale/C/N=text0/F=0/X=101.38/Y=2.34 image=M_Hist, lblMargin=-5
		AppendText "number of pixels"	
		
		// add controls
		
		Button StartROI,pos={10,35},size={70,20},proc=iS_MyRoiDrawButtonProc,title="Start ROI"
		Button clearROI,pos={90,35},size={80,20},proc=iS_MyRoiDrawButtonProc,title="Delete ROI"
		Button GetPixels,pos={180,35},size={70,20},proc=iS_MySaveRoiCopyProc,title="Get Pixels"		
		Button MakeGroup,pos={260,35},size={90,20}, proc = iS_MakeGroupFromIntensitiesCtrl, title="Make Group"	
		Slider MaxSlider, pos={30,15},size={200,15} ,value = 1, side= 0,vert= 0, limits={0.001, 1, 0}, proc=iS_SliderMaxDisplay				
	else
		//if window exists update annotation 
		Label left "\\Z16  counts " +yElem
		Label top "\\Z16  counts "+xElem
	endif
	
	return 1
		
end


Function iS_SliderMaxDisplay(sa) : SliderControl
	STRUCT WMSliderAction &sa	
	
	String ImageHost = sa.win
	String Image = StringFromList(0,ImageNameList(ImageHost, ";"), ";")
	wave w = ImageNameToWaveRef(ImageHost, image)
	
	switch( sa.eventCode )
		case -1: // control being killed
			break
		default:
				
			ControlInfo /W=$ImageHost MaxSlider
			variable SV = V_Value
			
			variable wMax = WaveMax(w)
			variable MaxDisplay = trunc(SV * wMax)
			
			// max should not be smaller than 2
			if(MaxDisplay < 2)
				MaxDisplay = 2
			endif
	
			ModifyImage /W=$ImageHost $NameOfWave(w) ctab= {1,(MaxDisplay),Terrain,0}
			
			break
	endswitch

	return 1
End


ThreadSafe function iS_CantorPF(aa, bb)
	variable aa, bb
	
	variable sumTerm  = aa + bb
	return  1/2 * (sumTerm) * (sumTerm + 1) + bb
end


function/Wave  iS_PixFromIntensities(xw, yw, M_Hist, M_ROIMask, M_ActivePhases)   // returns wave reference 
	wave xw, yw, M_Hist, M_ROIMask, M_ActivePhases
	
	string dfsave = GetDataFolder(1)
	SetDataFolder IntPlotData
		
	Duplicate/FREE/O xw xx
	Duplicate/FREE/O yw yy	
	
	variable rows = DimSize(M_ActivePhases, 0)
	variable cols = DimSize(M_ActivePhases, 1)	
	
	MatrixOP/FREE/O M_PixHistROI = M_Hist * M_ROIMask

	// Only look at Pixels/phases which are set on "Show" 
	Multithread xx = M_ActivePhases == 1 ? xx : 0
	Multithread yy = M_ActivePhases == 1 ? yy : 0	
	
	variable rowsHist = DimSize(M_PixHistROI , 0)
	variable colsHist = DimSize(M_PixHistROI , 1)
	
	Make/FREE/O/N=(rowsHist * colsHist) nPix, countsX, countsY
	Multithread nPix = M_PixHistROI[p]
	Multithread countsY = floor(p / rowsHist )
	Multithread countsX = mod(p, rowsHist)
	//Multithread countsX = p - trunc(p / rowsHist ) * rowsHist
		
	Multithread countsX = nPix == 0 ? 0 : countsX
	Multithread countsY = nPix == 0 ? 0 : countsY
	sort nPix, nPix, countsX, CountsY
	
	Multithread nPix = nPix == 0 ? NaN : nPix
	
	variable MinCount = WaveMin(nPix)
	Findlevel/P/Q nPix, MinCount
	DeletePoints 0, (V_LevelX), nPix, countsX, countsY 	
	
	variable nXYpairs = numPnts(nPix)
	
	// make wave to keep results vom Cantor function
	Make/FREE/O/I/U/N=(nXYPairs) ROISolutions 
	Make/FREE/O/I/U/N=(rows * cols)  MapSolutions
	
	// Make result wave unsigned 8 bit to make it compatible with ImageMorphology
	Make/O/B/U/N=(rows * cols)M_PixFromDensityPlot = 0  

	// make unique integers using cantor pairing function
	MultiThread ROISolutions = iS_CantorPF(countsX[p], countsY[p])
	MultiThread MapSolutions = iS_CantorPF(xx[p], yy[p])
	
	// make a point order wave and sort integers 
	Make/FREE/O/N=(numPnts(MapSolutions)) PntOrder = p
	Sort MapSolutions MapSolutions, PntOrder
	Sort ROISolutions ROISolutions

	// clip everything outside the WaveMin - WaveMax range in the map wave
	FindLevel/Q/P MapSolutions,  ROISolutions[0] // minimum						//WaveMin(ROISolutions)
	DeletePoints 0, V_LevelX, MapSolutions, PntOrder	
	FindLevel/Q/P MapSolutions, 	ROISolutions[(numpnts(ROISolutions)-1)]			//WaveMax(ROISolutions)+1
	DeletePoints V_LevelX, inf, MapSolutions, PntOrder
	
	variable np = numpnts(ROISolutions)
	variable length = numpnts(MapSolutions)
	Make/FREE/O/N=(length) tmp=NaN
	

	//Find values from ROI in wave
	// BinarySearchInterp returns the point number if value is present in a wave or a fractional, interpolated pointnumber if value is not present
	// use Mod(value, 1) to determine whether return value has a remainer when devided by 1   
	Multithread tmp = mod(binarySearchInterp(ROISolutions, MapSolutions[p]), 1) == 0 ? 1 : NaN
	
	// eliminate points outside ROI and condense wave
	Multithread PntOrder *= tmp
	WaveTransform zapNaNs PntOrder

	// Fill resulting wave in 1D with 1 for pixels in ROI
	variable i
	for (i=0; i<numPnts(PntOrder); i+= 1)		
		M_PixFromDensityPlot[PntOrder[i]] = 1 
	endfor
	
	// make 2D image from 1D wave	
	Redimension /N=(rows,cols) M_PixFromDensityPlot
		
	SetDataFolder dfsave
	
	Return M_PixFromDensityPlot
end



Function iS_MySaveRoiCopyProc(ctrlName) : ButtonControl		// called when Get Pixels is pressed
	String ctrlName
	
	String topwave = "root:IntPlotData:M_Hist"
	
	WAVE/Z ww=$topWave
	wave/Z  M_BaseImage = root:packages:iSpectra:M_BaseImage
	
	if(!WaveExists(ww) || !WaveExists(M_BaseImage))
		return 0
	endif
	
	String saveDF=GetDataFolder(1)
	String waveDF=GetWavesDataFolder(ww,1 )
	SetDataFolder waveDF
	
	ImageGenerateROIMask $WMTopImageName()
	
	SetDataFolder saveDF
	
	// now get Pixels out of it	
	dfRef dfr =  root:Packages:iSpectra:	
	wave M_SumImage = root:M_SumImage
	SVAR xElem = dfr:xElem
	SVAR yElem = dfr:yElem
	
	//make and get wave reference to M_Int
	wave M_PixFromDensityPlot = iS_PixFromIntensities($"root:Intensities:"+xElem, $"root:Intensities:"+yElem, root:IntPlotData:M_Hist, root:IntPlotData:M_ROIMask, root:IntPlotData:M_ActivePhases)
	iS_ScaleIfMagIsKnown(M_PixFromDensityPlot)
	

	DoWindow IntPixels
	if(V_flag)
		DoWindow/F IntPixels
		GraphNormal /W=IntPixels     //returns to normal model: no editing of poly
	else
		NewImage/K=1 /N=IntPixels M_BaseImage
		AppendImage M_PixFromDensityPlot
		SetWindow IntPixels,hook(s)= Hook
	endif
	
	ModifyImage M_PixFromDensityPlot explicit=1,eval={0,-1,-1,-1},eval={1,0,65535,0}
end



Function iS_MyRoiDrawButtonProc(ctrlName) : ButtonControl  // enable drawing
	String ctrlName

	DoWindow/F biHist
	if (V_Flag == 0)
		return 0
	endif
	
	
	String ImGrfName= WMTopImageGraph()

	if( strlen(ImGrfName) == 0 )
		return 0
	endif

	DoWindow/F $ImGrfName
	
	strswitch(ctrlname)	// string switch
		
		case "StartROI":		// execute if case matches expression
			ShowTools/A poly
			SetDrawLayer ProgFront
			Wave w= $WMGetImageWave(ImGrfName)		// the target matrix
			String iminfo= ImageInfo(ImGrfName, NameOfWave(w), 0)
			String xax= StringByKey("XAXIS",iminfo)
			String yax= StringByKey("YAXIS",iminfo)
			SetDrawEnv linefgc= (65535,65535,0),fillpat= 0,xcoord=$xax,ycoord=$yax,save
						
			break						
		case "clearROI":		
			GraphNormal
			SetDrawLayer/K ProgFront
			SetDrawLayer UserFront
			KillWaves/Z root:IntPlotData:M_ROIMask
			
			break		
	endswitch
End




Function iS_MakeGroupFromIntensitiesCtrl(ctrlName) : ButtonControl				 //Executed when "Create Group" is clicked
	String ctrlName
	
	wave/Z M_Groups = root:M_groups
	wave/Z M_SumImage = root:M_SumImage
	
	DFREF dfr =  root:packages:ispectra:
	wave/Z/T M_Model = dfr:M_Model
	
	if (!WaveExists(M_Groups) || !WaveExists(M_Model) || !WaveExists(M_SumImage) )
		return 0
	endif
	
	iS_MakeGroupFromIntensities()	
	
	iS_UpdatePixelCount()
	iS_iSpectraGroupPanel()
	iS_DisplayGroupsOnGrey(M_Model, M_Groups, M_SumImage)		//run to update color on M_Groups image	
end


Function iS_MakeGroupFromIntensities()
	
	wave/Z M_Groups = root:M_groups
	wave/Z M_SumImage = root:M_SumImage
	
	DFREF dfr =  root:packages:ispectra:
	wave/Z/T M_Model = dfr:M_Model
	wave/Z/T  History = dfr:History
	wave/Z Settings = dfr:Settings
	SVAR/Z xElem = dfr:xElem
	SVAR/Z yElem = dfr:yElem
	SVAR/Z ActivePhases = dfr:ActivePhases
	
	if (!WaveExists(M_Groups) || !WaveExists(M_Model) || !WaveExists(Settings) || !WaveExists(M_SumImage) ||  !SVAR_Exists(xElem)  || !SVAR_Exists(yElem) )
		return 0
	endif
	
	if (!DataFolderExists("root:IntPlotData:" ) )
		return 0
	endif
	
	wave/Z M_Hist = root:IntPlotData:M_Hist
	wave/Z M_ActivePhases = root:IntPlotData:M_ActivePhases
	wave/Z M_ROIMask = root:IntPlotData:M_ROIMask
	wave/Z M_PixFromDensityPlot = root:IntPlotData:M_PixFromDensityPlot
	
	if (!WaveExists(M_Hist) || !WaveExists(M_ActivePhases) || !WaveExists(M_ROIMask) )
		return 0
	endif

	if (!waveExists(M_PixFromDensityPlot ))
		wave M_PixFromDensityPlot = iS_PixFromIntensities($"root:Intensities:"+xElem, $"root:Intensities:"+yElem, M_Hist, M_ROIMask, M_ActivePhases)
	endif
	
	
	// add new group to M_Model
	variable nGroups = DimSize(M_Model,0)
	InsertPoints /M=0 nGroups, 1, M_Model  
	M_model[nGroups][%Group] = num2str(nGroups) 
	M_model[nGroups][%R] = num2str(0)
	M_model[nGroups][%G] = num2str(65335)
	M_model[nGroups][%B] = num2str(0)
	M_model[nGroups][%List] = yElem +"_vs_" + xElem + "_" + num2str(Settings[%nIntPlots]) + ","   // Settings[%nIntPlots] adds a consecutive number to avoid name conflicts
	M_model[nGroups][%Show] = num2str(1)
	
	// set a consecutive number for the density plot groups
	Settings[%nIntPlots] +=1
	
	// now change M_groups	
	MultiThread M_Groups = M_PixFromDensityPlot == 1 ? (nGroups) : M_Groups
	
	//write to History
	String cmd = ""
	sprintf cmd, "ELEMENTS:%s,%s*PHASES:%s*ROWS:%s*COLS:%s*WAVE:", xElem, yElem, ActivePhases, (num2str(DimSize(M_ROIMask, 0))), (num2str(DimSize(M_ROIMask, 1))) 
	cmd += iS_BinMatrix2String(M_ROIMask)
	iS_WriteString2History(cmd)

End


// *************************************************************************************************
// ---------------------------------  Ternary         --------------------------------------
// *************************************************************************************************

Function iS_IntensityTernary()	
	string DFsave = getDataFolder(1)
	NewDataFolder/O/S root:IntPlotData
	Make/I/U/O/N=(500,500) M_TernHist = 0
	
	DoWindow/F TernaryHist
	if(V_flag ==0)
	
		Display/K=1 /N= TernaryHist
		AppendImage/T M_TernHist
		//ModifyImage M_TernHist ctab= {*,1,Magenta,0}
		ModifyImage M_TernHist ctab= {1,1,Terrain,0}
		ModifyImage M_TernHist minRGB=(0,0,0),maxRGB=0
		
		ModifyGraph noLabel=2,axThick=0, margin = 30, margin(left)=90, width = {Aspect, 1}, height = {Aspect, 1}
		SetAxis/R left 0, 86.8
		SetAxis top 0, 100
		
		Tag /W= TernaryHist /N=Abox/F=0/S=3/A=MT/X=-1/Y=-1/L=0 M_TernHist, 0, "A"
		Tag /W= TernaryHist /N=Bbox/F=0/S=3/A=MT/X=1/Y=-1/L=0 M_TernHist, 99, "B"
		Tag /W= TernaryHist /N=Cbox/F=0/S=3/A=MB/X=0/Y=0.5/L=0 M_TernHist, 90050, "C"
		
		//Get Wavelist in Intensities
		PopupMenu A, Title = "A",  pos = {10, 35}, value = iS_ReturnElements()
		PopupMenu B, Title = "B",   pos = {10, 60}, value =  iS_ReturnElements()
		PopupMenu C, Title = "C",  pos = {10, 85},  value =  iS_ReturnElements()
		Button PlotTernary, Title = "Do it", pos = {10, 110}, proc = iS_MakeTernaryHistWave
		Slider MaxSlider, pos={30,5},size={190,15} ,value = 1, side= 0,vert= 0, limits={0.001, 1, 0}, proc=iS_SliderMaxDisplay
		
		// draw ternary	
		variable thick = 2
		SetDrawEnv xcoord= top, ycoord= left, linefgc= (65535,65535,65535),linethick= Thick
		DrawLine 0,0,50,87
		SetDrawEnv xcoord= top, ycoord= left, linefgc= (65535,65535,65535),linethick= Thick
		DrawLine 50,87,99.5,0
		SetDrawEnv xcoord= top, ycoord= left, linefgc= (65535,65535,65535),linethick= Thick
		
	endif
	SetDataFolder dfsave
end

function/S iS_ReturnElements()
	string dfsave = GetDataFolder(1)
	
	if(!DataFolderExists("root:Intensities:"))
		return "error"
	endif
		
	SetDataFolder root:Intensities:
	string elements = WaveList("*", ";", "")
	SetDataFolder dfsave
		
	return elements
end


function iS_MakeTernaryHistWave(ctrlname) :ButtonControl 
	string ctrlname
	
	ControlInfo /W=TernaryHist A
	String sA = S_Value
	ReplaceText /W = TernaryHist /N=ABox sA
	
	ControlInfo /W=TernaryHist B
	String sB = S_Value
	ReplaceText /W = TernaryHist /N=BBox sB
	
	ControlInfo /W=TernaryHist C
	String sC = S_Value
	ReplaceText /W = TernaryHist /N=CBox sC
	
	Duplicate/FREE/O $"root:Intensities:" + sA, AA
	Duplicate/FREE/O $"root:Intensities:" + sB, BB
	Duplicate/FREE/O $"root:Intensities:" + sC, CC
	
	variable rows = DimSize(AA, 0)
	variable cols = DimSize(AA, 1)
	variable pnts = rows * cols
	
	Redimension/D/N=(rows*cols) AA, BB, CC
	Make/FREE/O/D/N=(rows*cols) XX, YY
	Make/O/FREE/D/N=(rows*cols) total
	
	total = AA + BB + CC
	
	AA /= total
	BB /= total
	CC /= total
	
	XX = 100 * (0.5 * CC + BB)			//Do the coordinate transformation
	YY = 100* ( CC * cos(30*(Pi/180)))
	
	wave /Z M_TernHist = root:IntPlotData:M_TernHist
	M_TernHist = 0
	
	//which groups are shown?
	iS_MaskOfActiveGroups(root:IntPlotData:)
	wave M_ActivePhases =  root:IntPlotData:M_ActivePhases
	
	MultiThread XX *= M_ActivePhases
	MultiThread YY *= M_ActivePhases
	
	MultiThread XX = XX == 0 ? NaN : XX
	MultiThread YY = YY == 0 ? NaN : YY
	
	Variable  xpnt, ypnt,  dx, dy,x0, y0
	dx = DimDelta(M_TernHist, 0)
	dy = DimDelta(M_TernHist, 1)
	x0 = DimOffset(M_TernHist, 0)
	y0 = DimOffset(M_TernHist, 1)
	
	
	Variable maxXpnt = dimSize(M_TernHist, 0)
	Variable maxYpnt = dimSize(M_TernHist, 1)
	
	variable i
	for (i = 0; i < pnts; i += 1)
		if (  (numtype(XX[i]) != 0) || (numtype(YY[i]) != 0) )		// avoid NaN and Inf
			continue
		endif
		
		xpnt = floor((XX[i]-x0)/dx)
		ypnt = floor((YY[i]-y0)/dy)
	
		if ( (xpnt >= maxXpnt) || (ypnt >= maxYpnt))
			continue
		endif		
		M_TernHist[xpnt][ypnt]+= 1	
	endfor
	
	DoWindow/F TernaryHist
	ModifyImage /W=TernaryHist $NameOfWave(M_TernHist) ctab= {1,*,Terrain,0}
	ModifyImage M_TernHist minRGB=(0,0,0),maxRGB=0
end



// generates M_ActivePhases 
function iS_MaskOfActiveGroups(folder)
	DFREF folder
	
	dfRef dfr =  root:Packages:iSpectra:	
	
	wave/Z/T M_Model =dfr:M_Model
	wave/Z cube = root:cube
	wave/Z M_Groups = root:M_Groups
	
	if (!WaveExists(M_Model) || !WaveExists(cube) || !WaveExists(M_Groups))
		return 0
	endif
	
	Make/B/U/O/N=(Dimsize(cube,0), DimSize(cube,1)) folder:M_ActivePhases=0
	wave M_ActivePhases =  folder:M_ActivePhases
		
	variable i
	for(i=0; i<DimSize(M_Model, 0); i+=1)
		
		if(str2num(M_model[i][%Show]) == 0)		//if phase not shown, skip it
			continue
		endif	
		
		MultiThread M_ActivePhases += M_Groups == i ? 1 : 0
	endfor	
end


// *************************************************************************************************
// ---------------------------------   History functions    ----------------------------------
// *************************************************************************************************

function iS_WriteString2History(str)
	string str
	
	wave/T/Z History = root:packages:iSpectra:History
	if(!WaveExists(History))
		Abort "WaveExists() Error."
	endif
		
	variable pos = numpnts(History)
	InsertPoints (pos), 1, History
	History[pos] = str
end

function iS_SaveHistory()	

	wave/T/Z History = root:packages:ispectra:History
	wave/T/Z M_Model = root:packages:ispectra:M_Model
	
	if (!WaveExists(History)|| !WaveExists(M_Model))
		return 0
	endif
	
	//write M_Model to History
	variable i
	for(i=0; i<DimSize(M_Model, 0); i+=1)
		variable pos = numpnts(History)
		InsertPoints (pos), 1, History
		string line = ""
		Line += "M_Model: *LIST:" + M_Model[i][%List]
		Line += "*RED:" + M_Model[i][%R]
		Line += "*GREEN:" + M_Model[i][%G]
		Line += "*BLUE:" + M_Model[i][%B]
		Line += "*PHASE:" + M_Model[i][%Phase]
			
		History[pos] = line
	endfor
		
	Save/T History
end

function iS_ApplyHistory()
	
	Wave/Z cube = root:cube
	if (!WaveExists(cube))
		DoAlert, 0, "First you must load a spectral image!"
		return 0
	endif
	
	wave/T/Z w = root:history
	if(!WaveExists(w))
		LoadWave/T /O
		if (V_Flag==0)
			return 0
		endif
		
		wave/T/Z w =  $RemoveEnding(S_WaveNames, ";") // takes ";" away
		
	endif
	
	// is loaded wave a history wave?
	if(cmpstr(w[0],"iSpectraHistory:") != 0)
		DoAlert 0, "This is no valid History file"
		return -1
	endif
	
	DoAlert 1, "Warning: All pixel assignments and settings will be reset. Do you want to continue?"
	if(V_Flag == 2)
		return 2
	endif
	
	iS_ApplyHistoryWave(w)
	iS_UpdatePixelCount()
	iS_iSpectraGroupPanel()
	iS_OpenGroupsOnGrey()
end




function iS_ApplyHistoryWave(hist)
	wave/Z/T hist
	
	if (!WaveExists(hist))
		return 0
	endif
	
	KillWaves/Z M_Groups
//	iS_Initialise()
	
	//reset history wave if there is one
	wave/T/Z oldHistory = root:packages:iSpectra:History
	if(WaveExists(oldHistory))
		DeletePoints 3, inf, OldHistory
	endif
	
		
	DFREF dfr = root:packages:ispectra
	wave/Z Settings = dfr:Settings
	
	// set counter for density plots to zero - may cause trouble otherwise
	Settings[%nIntPlots] = 0
	
	
	
	//start executing commands or fill parameters in settings wave
	variable i,j, length = numpnts(hist)
	for (i=1; i<length; i+=1)
		
		string line = Hist[i]
		string WhatToDo = StringFromList(0, line, ":")
		string ThisCmd = ""
		
		StrSwitch(WhatToDo)
		
			// update AccV
			case "AccV":				
				Settings[%AccV] = str2num(StringByKey("AccV", line))
				break
		
		
			// get measurement time per pixels
			case "TimePerPix":
				Settings[%DwellTimePerPixel] = str2num(StringByKey("TimePerPix", line))
				break
				
			//update Magfification	
			case "Magnification":
				Settings[%Magnification] = str2num(StringByKey("Magnification", line))
				break				
				
		
			// update lineRef
			case "LINEREFELEMENTS":
				string LineRefElements = StringByKey("LINEREFELEMENTS", line, ":", "*")
				Make/O/N=(ItemsInList(LineRefElements))  dfr:LineRef
				wave LineRef = dfr:LineRef
				for (j=0;j<ItemsInList(LineRefElements);j+=1)
					SetDimLabel 0,j,$StringFromList(j,LineRefElements) , LineRef
				endfor
				
				break
			
			case "LINEREFENERGIES":
				string LineRefEnergies = StringByKey("LINEREFENERGIES", line, ":", "*")
				wave LineRef = dfr:LineRef
		
				for (j=0;j<ItemsInList(LineRefEnergies);j+=1)
					LineRef[j] = str2num(StringFromList(j, LineRefEnergies, ";"))
				endfor
				
				break
	
	
			
			// this is a command that can be executed directly from the command line
			case "CMD" :
			
				ThisCmd = StringByKey("CMD", line)
				execute ThisCmd
				
				break
			
			
			// this case handels density plot events
			case "ELEMENTS" :
				
				string elements = StringByKey("ELEMENTS", line, ":", "*")
				string xElem = StringFromList(0, elements, ",")
				string yElem = StringFromList(1, elements, ",")
				string activePhases =  StringByKey("PHASES", line, ":", "*")
				string w2Dstr =  StringByKey("WAVE", line, ":", "*")
				variable  rows = str2num( StringByKey("ROWS", line, ":", "*") )
				variable cols = str2num( StringByKey("COLS", line, ":", "*") )
		
		
				// get wave references and make histogram
				wave xx = $"root:Intensities:"+xElem
				wave yy = $"root:Intensities:"+yElem
		
				//update global strings
				SVAR/Z gX = root:Packages:iSpectra:xElem; gX = xElem
				SVAR/Z gY = root:Packages:iSpectra:yElem; gY = yElem
				
				variable check = iS_MakeHistogram(xx, yy, activePhases)
				if(check != 1)
					// if check is not 1, then intensity plot was unsuccesful
					break
				endif
				
				wave M_ActivePhases = root:IntPlotData:M_ActivePhases
				wave M_Hist = root:IntPlotData:M_Hist
				
				// Folder IntPlotData exists, now create M_ROIMask
		
				Make/I/U/O/N=(rows*cols) root:IntPlotData:M_ROIMask
				wave M_ROIMask =  root:IntPlotData:M_ROIMask
				MultiThread M_ROIMask = str2num(w2Dstr[p])
				Redimension/N=(rows, cols) M_ROIMask
				

				//wave M_ROIMask = string2Matrix("root:IntPlotData:", w2dstr)
				// redimension mask to avoid dimension mismatch error
				Redimension/N=(DimSize(M_Hist, 0), DimSize(M_Hist, 1)) M_ROIMask
		
				iS_MakeGroupFromIntensities()
				KillDataFolder/Z root:IntPlotData
				
				break
			
			// this case updates M_Model with previous colors and phase names	
			case "M_Model" :
			
				wave/T/Z M_Model = dfr:M_Model
				string ThisGroup = StringByKey("LIST", line, ":", "*")
									
				variable start2look = 0
				for(j=0; j<DimSize(M_Model, 0); j+=1)
					
					// is there an identical group in present M_Model?; if yes, update color and name	

					if (cmpstr(ThisGroup, M_Model[j][%List]) == 0)
					
						M_Model[j][%Phase] =  StringByKey("PHASE", line, ":", "*")
						M_Model[j][%R]  =  StringByKey("RED", line, ":", "*")
						M_Model[j][%G]  =  StringByKey("GREEN", line, ":", "*")
						M_Model[j][%B]  =  StringByKey("BLUE", line, ":", "*")
						break
					endif		
				endfor



			break
				
		endswitch
		
	endfor
end



Function/S iS_BinMatrix2String(w2D)    // Make a String from a wave
	wave w2D
	
	variable rows = DimSize(w2D,0)
	variable cols = DimSize(w2D,1)
	
	Duplicate/O/FREE w2D, tmp
	Redimension/N=(rows*cols) tmp   
	
	String w2DStr = ""
	
	variable i
	for(i=0; i<(numpnts(w2D)); i+=1)
		w2Dstr += num2str(w2D[i])
	endfor
	
	return w2DStr
end 


function iS_MergeUnNamedGroups(minPixPerc, mode)
	variable minPixPerc, mode						
	// if mode == 1 unnamed groups will be merged with trash if isParticle() return 0
	
	wave/T/Z M_Model =root:packages:ispectra:M_Model
	
	if (!WaveExists(M_Model))
		return 0
	endif
	
	iS_UpdatePixelCount()
	
	variable i = 1
	variable nPixels =0, nGroups = 0, counter=0
	
	do
	
		String phase = M_Model[i][%Phase]
		String elements = M_Model[i][%List]
		variable PixPerc = str2num(M_Model[i][%PixPercent])
		
		if ( (strlen(phase) == 0))
			// deal with un-named groups
			
			if (PixPerc < minPixPerc) 
				iS_Merge("TRASH,", elements)
				nPixels+=PixPerc
				nGroups+=1
				i-=1
			else
				
				M_Model[i][%Phase] = "UnknownPhase_"+num2str(counter) 
				M_Model[i][%R] = "65535"
				M_Model[i][%G] = "65535"		
				M_Model[i][%B] = "65535"
				counter +=1
				
				if (mode==1)
					if (iS_isParticle(i, minParticleArea) == 0)
						
						iS_Merge("TRASH,", M_Model[i][%list])
						nPixels+=PixPerc
						nGroups+=1
						i-=1
						counter -=1
						//	printf "ATTENTION: %s is a particle!\r",  M_Model[i][%Phase]					
					else
						printf "ATTENTION: %s - %s - is a particle!\r",  M_Model[i][%Phase], M_Model[i][%list]
					endif		
						
				endif
			endif
		endif
		i+=1
		 
	while(i<DimSize(M_Model, 0))
	
	Printf "ATTENTION: %g groups with %g pixel percent were merged with TRASH \r", nGroups, nPixels
		
	// update counts and panel	
	iS_UpdatePixelCount()
	iS_iSpectraGroupPanel()	
end





// *************************************************************************************************
// --------------------------------- Visualization: Gradient and RGB functions------------------------------------
// *************************************************************************************************

function iS_Plotgradient()
	variable group
	string expression
	
	if (!DataFolderExists("root:Intensities:"))
		return 0
	endif
	
	Prompt group, "Enter group number: "
	Prompt expression, "Enter Intensity ratio: "
	DoPrompt "Select Group and enter ratio", group, expression
	
	if (V_flag)
		return -1
	endif
	
	iS_ShowGradient(group, expression)	
end		


function iS_ShowGradient(group, expression)
	variable Group
	string expression
	
	wave/Z cube = root:cube
	wave/Z M_BaseImage = root:Packages:iSpectra:M_BaseImage
	wave/Z M_groups = root:M_Groups
	
	
	if (!WaveExists(cube) ||  !WaveExists(M_BaseImage)  ||  !WaveExists(M_Groups)  ||  !DataFolderExists("root:Intensities:"))
		return 0
	endif
	

	SetDatafolder root:Intensities		
	Make/O/N= (DimSize(cube, 0), DimSize(cube, 1) ) M_Gradient
	iS_ScaleIfMagIsKnown(M_Gradient)
	string cmd = "M_gradient = " + expression
	execute cmd
	
	// print to History
	printf "ShowGradient(%g, \"%s\")\r", Group, expression

	Duplicate/O M_Gradient, root:M_Gradient
	KillWaves/Z M_Gradient
	
	Setdatafolder root:
	
	Wave M_Gradient = root:M_Gradient
	M_Gradient = M_Groups == group ? M_Gradient : NaN
		
	//display  M_gradient as overlay over M_BaseImage
	DoWindow Gradient
	if (V_flag)
		DoWinDow/F Gradient
	else	
		NewImage/K=1/N=Gradient M_BaseImage
		AppendImage/W= Gradient M_Gradient
		SetWindow Gradient,hook(s)= iS_Hook
		ModifyImage M_Gradient ctab= {*,*,YellowHot,0}
		ColorScale/C/N=text0/S=3 /X=105 /Y=5.5 image=M_Gradient, lblMargin=-5
		ColorScale/C/N=text0 expression + " count ratio"
		ModifyGraph margin(right)=100
	endif
end


function iS_GenerateRGBMap(ctrlName,popNum,popStr) : PopupMenuControl				
	String ctrlName
	Variable popNum
	String popStr
	
	ControlInfo /W=ModelSetup red
	String red = S_Value
	
	ControlInfo /W=ModelSetup green
	String green = S_Value
	
	ControlInfo /W=ModelSetup blue
	String blue = S_Value
	
	if (cmpstr(red, "red")==0 || cmpstr(green, "green")==0  || cmpstr(blue, "blue")==0)
		return 0
	endif
	
	string dfsave = GetDataFolder(1)
	SetDataFolder root:Intensities:
	iS_RGBGenerator(red, green, blue)
	wave M_RGBImage
	iS_ScaleIfMagIsKnown(M_RGBImage)
	
	Duplicate/O M_RGBImage root:M_RGBImage
	KillWaves/Z M_RGBImage
	SetDataFolder $dfsave 
	wave M_RGBImage
	
	
	// Is there an RGB image displayed on the BaseImage
	DoWindow BaseImage
	if(V_flag) 
		ControlInfo/W=ModelSetup ImageSelector
		if(cmpstr(S_value, "M_RGBImage") == 0)
			//RGB Image is schown - update it
			wave M_BaseImage = root:packages:ispectra:M_BaseImage
			M_BaseImage = M_RGBImage
			return 0
		endif
	endif
		
	
	DoWindow M_RGBMap
	if(V_flag==0)	
		NewImage/K=1/N=M_RGBMap M_RGBImage
		SetWindow M_RGBMap,hook(s)=iS_Hook
	endif
	
	return 1
end

function iS_RGBGenerator(R, G, B)
	string R, G, B
	
	wave/Z red = $R
	wave/Z green = $G
	wave/Z blue = $B
	
	if(WaveExists(red))
		Make/W/U/O/N= (DimSize(red, 0), DimSize(red, 1), 3) M_RGBImage = 0
		Make/W/U/O/N= (DimSize(red, 0), DimSize(red, 1)) M_tmp
	else
		DoAlert 0, "Can't generate image!"
		return 0
	endif
	
	// "execute" allows expressions, such as e.g. "Si/Al", to be used as RGB channels
	string cmd = "M_tmp =" + R
	Execute cmd
	iS_scaleImageTo16Bit(M_tmp)	 
	M_RGBImage[][][0] = M_tmp[p][q]
	
	cmd = "M_tmp =" + G
	Execute cmd
	iS_scaleImageTo16Bit(M_tmp)	
	M_RGBImage[][][1] = M_tmp[p][q]
	
	cmd = "M_tmp =" + B
	Execute cmd
	iS_scaleImageTo16Bit(M_tmp)	
	M_RGBImage[][][2] = M_tmp[p][q]
	 
	KillWaves/Z M_tmp
	return 1
end


function iS_MakeGroupTif(Group, ImageMatrix, outname)
	variable group
	wave ImageMatrix
	string OutName
	
	wave/T/Z M_Model = root:packages:iSpectra:M_Model
	wave/Z M_groups = root:M_Groups
	
	if(!WaveExists(ImageMatrix) || !WaveExists(M_Model)  || !WaveExists(M_Groups))
		Abort "WaveExists() Error"
	endif
	
	Make/O/W/U/N=(DimSize(ImageMatrix, 0), DimSize(ImageMatrix, 1), 3) $outname
	wave w = $outname
	iS_ScaleIfMagIsKnown(w)
	
	// produce base image
	if(DimSize(ImageMatrix,2) == 2)
		w=ImageMatrix
	else
		w[][][0,2] = ImageMatrix[p][q]
	endif
	
	//make group overlay
	variable red = str2num(M_Model[group][%R])
	variable green = str2num(M_Model[group][%G])
	variable blue = str2num(M_Model[group][%B])
	
	w[][][0] = M_Groups[p][q] == group ? red : w 
	w[][][1] = M_Groups[p][q] == group ? green : w 
	w[][][2] = M_Groups[p][q] == group ? blue : w 
	
	return 1
end


function iS_SelectiveRGB(image)
	wave/Z image
	
	wave/T/Z M_Model =root:packages:ispectra:M_Model
	wave/Z M_RGBImage = root:M_RGBImage
	wave/Z M_Groups = root:M_Groups
	
	if (!WaveExists(M_Model)|| !WaveExists(M_RGBImage)|| !WaveExists(M_Groups) || !WaveExists(image))
		return 0
	endif
	
	Make/FREE/W/U/O/N=(DimSize(M_Groups,0),DimSize(M_Groups,1)) M_Mask = 0

	variable i
	for(i=0; i<DimSize(M_Model, 0); i+=1)
		if(str2num(M_Model[i][%Show]) == 1)		
			MultiThread M_Mask = M_Groups == i ? 1 : M_Mask
		endif
	endfor
	
	Duplicate/O M_RGBImage M_RGBSelective
	
	MultiThread M_RGBSelective[][][0,2] = M_Mask[p][q] == 0 ? image[p][q] : M_RGBSelective
	end
end



// *************************************************************************************************
// --------------------------------- Cross correlation        ----------------------------------------------
// *************************************************************************************************



Function iS_GetCrossCorrelation(cube, M_Groups, threshold) 
	wave cube, M_Groups
	variable threshold
	
//	variable start = StartMStimer
	
	if(!WaveExists(cube) || !WaveExists(M_Groups) )
		return 0
	endif
	
	wave/Z M_GroupsErodeUndo = root:M_GroupsErodeUndo
	If (WaveExists(M_GroupsErodeUndo))
		DoAlert 0, "First turn Erosion off!"
		return 0
	endif
	
		
	variable nRows = DimSize(M_Groups, 0)
	variable nCols =  DimSize(M_Groups, 1)	
	variable nLayers = DimSize(cube, 2)	
	variable nGroups = WaveMax(M_Groups) - WaveMin(M_Groups) + 1
	
	Make/FREE/O/N=(nLayers, nGroups) M_spectra
	Make/FREE/O/N=(nLayers ) EDS_tmp = 0
	Duplicate/O/FREE M_Groups, Mask   //create mask from M_Groups
		
	variable i	
	for(i=0; i<nGroups; i+=1)  // run through all groups .... 
	 	
	 	// get mask of pixels of interest
		MultiThread Mask = M_Groups == i ? 1 : 0
		
		// get EDS spectrum for each phase and normalise to abundance 
		variable sumPhasePixels = sum(Mask) 
		MultiThread EDS_tmp[0,(nChannelsForCrossCorrelation);  (IntervalForCrossCorrelation)] = iS_WorkerFunc(cube, Mask, p)/ sumPhasePixels
		
		// Fill matrix			
		MultiThread M_Spectra[][i] = EDS_tmp[p]		
	endfor
	
	MatrixOp /O M_CorrMatrix=syncCorrelation(M_Spectra) 	// calculate correlation matrix
	MatrixOp/FREE /O Variances=varcols(M_Spectra)
	MultiThread M_CorrMatrix /=sqrt(Variances[p]*Variances[q])
	MultiThread M_CorrMatrix = abs(M_CorrMatrix)
	MultiThread M_CorrMatrix = M_CorrMatrix >= Threshold ? M_CorrMatrix : NaN 		//set values smaller than threshold to NaN
	
	
	DoWindow CorrelationMatrix		//Display Matrix
	if (V_flag)
		DoWinDow/F CorrelationMatrix
	else	
		edit /K=1 /N= CorrelationMatrix M_CorrMatrix
		ModifyTable sigDigits(M_CorrMatrix)=3,width(M_CorrMatrix)=40
	endif	
	
	//print StopMStimer(start)/1e6
End



Function iS_CrossCorrelation(ctrlName) : ButtonControl
	String ctrlName
	
	wave/Z M_Groups = root:M_Groups
	wave/Z cube = root:cube
	wave/Z Settings = root:packages:iSpectra:Settings
	wave/Z M_Model = root:packages:iSpectra:M_Model
	
	if (!WaveExists(cube) || !WaveExists(M_Groups) ||  !WaveExists(Settings)  ||  !WaveExists(M_Model))
		DoAlert 0, "WaveExists() Error"
		return 0
	endif
	
	strswitch(ctrlName)	
	
		case "CrossCorrDoIt":		
			variable threshold = Settings[%vCorr]
			iS_GetCrossCorrelation(cube, M_Groups, threshold)
			break						
		case "ShowMatchesDoIt":	
			
			ControlInfo /W=ModelSetup ShowMatchesSetGroup
			iS_ShowRelatedGroups(V_Value)
			break

	endswitch
End


function iS_ShowRelatedGroups(group)
	variable group
	
	wave/Z M_CorrMatrix = root:M_CorrMatrix
	wave/T/Z M_Model = root:packages:iSpectra:M_Model
	
	if (!WaveExists(M_CorrMatrix) || !WaveExists(M_Model))
		DoAlert 0, "WaveExists() Error!"
		return 0
	endif
	
	variable MaxRow = DimSize(M_CorrMatrix, 0)
	variable nGroups = DimSize(M_Model, 0)
	
	if (MaxRow != nGroups)
		DoAlert 0, "Dimension mismatch!\r Run Cross Correlation again!"
		return -1
	endif
	
	// Kill an existing Window
	DoWindow/F SpectrumDisplay
	if (V_flag)
		KillWindow SpectrumDisplay
	endif
	
	// check which groups are likely identical and diplay all of them
	Variable i
	for(i=0; i< MaxRow; i+=1)
	
		//set Show checkbox to off
		M_Model[i][%Show]= num2str(0)
		
		// if NaN -> no correlation
		if(numtype(M_CorrMatrix[i][group]) == 2)
			continue
		endif
		
		iS_GetSpectrum(i)
		M_Model[i][%Show]= num2str(1)
		iS_ShowPhases(i, str2num(M_Model[i][%Show]))
		
	endfor 	
		
	// sync color and normalise spectra
	//SyncDisplayColors(root:Packages:iSpectra:M_Model, "SpectrumDisplay")
	//NormaliseSpectra(root:spectra, 1)
	
	//build group panel with groups of interest
	iS_iSpectraGroupPanel()
end




// *************************************************************************************************
// --------------------------------- background functions   ----------------------------------------------
// *************************************************************************************************



function iS_BKG_Panel() : Panel
	PauseUpdate; Silent 1		// building window...
	NewPanel/K=1 /N= BKGPanel/W=(1207,302,1446,358)
	Button ContinueBKG,pos={127,19},size={80,20},title="Continue", proc = iS_BKGPanelAction
	Button CancelBKG,pos={30,19},size={80,20},title="Cancel", proc = iS_BKGPanelAction
End

Function iS_BKGPanelAction(ctrlName) : ButtonControl
	String ctrlName
	
	strswitch(ctrlName)
		case "CancelBKG":			
			RemovefromGraph /Z /W= PointSpectrum ybkg, bkg
			KillWaves/Z ybkg, xbkg	
			DoWindow /K BKGPanel
			break
	
		case "ContinueBKG":

			 iS_ApplyBKG(root:xbkg, root:ybkg)
			DoWindow /K BKGPanel
			break
		
	endswitch	 
End


function iS_DrawBKGWave()	
	dfref dfr = root:Packages:iSpectra
	
	if (!WaveExists(dfr:CrsSpec))
		DoAlert 0, "Click on grey scale image to generate a spectrum"
		return 0
	endif
	
	wave CrsSpec = dfr:CrsSpec
		
	DoWindow/F PointSpectrum
	if (V_flag==0)
		Display/K=1/N=PointSpectrum CrsSpec		
	endif
	RemovefromGraph /Z /W= PointSpectrum ybkg, bkg
	
	iS_BKG_Panel()
	GraphWaveDraw /W=PointSpectrum /O ybkg, xbkg	
	PauseForUser BKGPanel, PointSpectrum 
	GraphNormal
end


function  iS_ApplyBKG(xbkg, ybkg)
	wave xbkg, ybkg
	
	dfref dfr = root:Packages:iSpectra
	wave CrsSpec = dfr:CrsSpec
	wave Settings = dfr:Settings
		
	Make/N=(numpnts(CrsSpec))/O bkg
	
	Interpolate2/T=1/N=(numpnts(CrsSpec))/Y=bkg xbkg, ybkg
	SetScale/I x, 0, (Settings[%AccV]), "", bkg
	
	DoWindow/F PointSpectrum
	RemoveFromGraph ybkg
	KillWaves /Z xbkg, ybkg	
	AppendToGraph bkg
	ModifyGraph rgb(bkg)=(0,0,65535)
	
	iS_subbkg(bkg, root:cube)
end


function iS_subbkg(bkg, cube)
	wave bkg, cube
	
	redimension/W cube
	variable timeref = startMStimer
	variable i
	
	for(i=0; i<numpnts(bkg); i+=1)
				
		MultiThread cube[][][i] -= bkg[i]		
	
	endfor
	
	printf "Timerequired to substract background: %f seconds\r", (stopMStimer(timeref) /1e6)
	iS_AnalyseCube(:cube, :Packages:iSpectra:LineRef, avgChannels) 
end

function iS_addbkg(bkg, cube)
	wave bkg, cube
	
	redimension/W cube
	variable timeref = startMStimer
	variable i
	
	for(i=0; i<numpnts(bkg); i+=1)
				
		MultiThread cube[][][i] += bkg[i]		
	
	endfor
	
	printf "Timerequired to substract background: %f seconds\r", (stopMStimer(timeref) /1e6)
	iS_AnalyseCube(:cube, :Packages:iSpectra:LineRef, avgChannels) 
end




// *************************************************************************************************
// --------------------------------- EMSA import export     ----------------------------------------------
// *************************************************************************************************


function  iS_EmsaHeaderLoad(filePath)		
	string filePath

	Make/T/O/N=0 root:EmsaHeader
	wave/T HeaderWave = root:EmsaHeader
	
	if(strlen(filePath)==0)
		variable refNum
		Open/D/R/F="*.emsa"/M="Select Extracted Spectrum" refNum
		filePath = S_fileName
	
		// is there a path or did user cancel?
		if(strlen(filePath) == 0)
			KillWaves/Z HeaderWave
		       return 0
		endif
	endif
		
	Open/R refNum as filepath
	
	variable i =0		
	do
		string line
		FReadLine refNum, line
	
		if( strlen(line) == 0 )							// end of file
			break										// exit do-while loop
		endif
			
		InsertPoints i, 1, HeaderWave
		
		// set dead time to zero
		if (StringMatch(line, "*DEADTIME*"))
			line = "##DEADTIME    : 0\r"
		endif
				
		HeaderWave[i] = line
		i+=1			
			
		if( stringmatch(line, "#SPECTRUM*")	== 1)	// energy range reached
			break										// exit do-while loop
		endif
	
	while( 1 )	// exit is through break statement
	Close refNum
	
	return 1
end

function iS_ExportSpectra(which, [folderPath, name])
	variable which 
	string folderPath
	string name
	
	//variable "which" determines if only selected spectra get exported (0) or all spectra (which !=0)
	
	DFREF dfr = root:packages:ispectra:
	wave/T/Z M_Model = dfr:M_Model	
	wave/Z cube = root:cube
	wave/Z M_Groups = root:M_Groups
	
	if (!WaveExists(M_Groups) ||  !WaveExists(cube)  ||  !WaveExists(M_Model) )
		return 0
	endif
	
	//check if a filename is parsed
	string optionalName=""
	if(!ParamIsDefault(name))
		OptionalName = name
	endif
	
	
	// Does HeaderWave Exist	
	wave/T/Z header = root:EmsaHeader	
	if ( !WaveExists(header) )
		iS_EMSAHeaderLoad("")	
		wave/T/Z header = root:EmsaHeader	
	endif
	
	if (ParamIsDefault(folderPath))
		// check if output path exists, if yes don't ask for
		PathInfo SpectrumPath
		if (!V_flag) 	
			NewPath/Q/O /M = "Select the output folder on disc" SpectrumPath   //select folder for spectra 
			PathInfo SpectrumPath
		endif
	else
		// export path is specified, called from FullSave function
		S_Path = folderPath
	endif
			
	string dfsave
	dfsave = GetDataFolder(1)
	NewDataFolder/O/S ExportTemp
	
	variable i
	for (i=0; i< DimSize(M_Model, 0); i+=1)
		
		if (which == 0)
			//if group is not selected, skip the rest of the loop; spectrum does not get exported	
			if (str2num(M_Model[i][%Show]) == 0)
				continue
			endif
		endif
		
				
		// use either given name or group name for spectrum
		string wName = ""
		if (strlen(M_Model[i][%Phase]) == 0)  // group has no name
			wName = "EDS_Group_" + num2str(i)
		else
			wName = M_Model[i][%Phase]
		endif
		
		Make/O/N=(DimSize(cube, 2)) $wName
		wave EDS = $wName
		
		Duplicate/O/FREE M_Groups, Mask   //create mask from M_Groups
		MultiThread Mask = M_Groups == i ? 1 : 0
		MultiThread EDS = iS_WorkerFunc(cube, Mask, p)	
		SetScale/P x, 0, (xPerChannel), "keV" EDS
		
		//if spectrum does not contain any counts, skip it
		if (sum(eds) == 0)
			continue
		endif
			
		// change livetime in header according to pixels in phase
		iS_ChangeTimesInHeader(sum(Mask))
		
		//call export function
		iS_ExportSpectrum(header, EDS, S_Path, OptionalName)
	endfor
		
	SetdataFolder dfsave
	KillDataFolder/Z ExportTemp
	
	if (!ParamIsDefault(folderPath))
		KillPath/Z SpectrumPath
	endif
end


function iS_ExportSpectrum(headerW, spectrum, pathString, OptionalName)
	wave/Z/T headerW
	wave/Z spectrum
	string PathString
	string OptionalName
	
	if (!WaveExists(HeaderW) ||  !WaveExists(spectrum) )
		DoAlert 0, "WaveExists() Error"
		return 0
	endif
	
	
	Variable refnum
	
	string filename=""
	if(strlen(OptionalName) == 0)
		fileName = NameOfWave(spectrum)
	else
		filename = NameOfWave(spectrum) +"_" + OptionalName
	endif
	
	Duplicate/O/Free  spectrum Energy
	Energy = x
	
	NewPath/Q/O SpectrumPath, pathString
	
	Open/Z=2 /P = SpectrumPath /T="emsa"  refnum   as FileName+".emsa"
	
	if (V_flag == -1)   //User cancelled dialog
		return -1
	endif
	
	wfprintf refnum, "%s\n", headerW				                  			  // writes header file to file, followed by return
	wfprintf refnum, "%2.4f, %.1f,\r\n",  Energy, spectrum			// writes energy and count data to file, followed by return
	fprintf refnum, "#ENDOFDATA   : \r\n"
	Close refnum
	
	return 1
end


function iS_ChangeTimesInHeader(PixelsPerPhase)
	variable PixelsPerPhase
	// this function adjusts live time, real time and dead time in EMSA header; depends on pixels per phase
	// valid here for NSS datasets converted with SIconvert.exe, which does dead time correction 
	
	
	DFREF dfr = root:packages:iSpectra:
	wave Settings = dfr:Settings
	wave/T HeaderWave = root:EmsaHeader
	
	variable LiveTime = Settings[%DwellTimePerPixel] * PixelsPerPhase
	variable RealTime = LiveTime
	variable DeadTime = 0
	variable i
	
	for(i=0; i<numpnts(headerWave); i+=1)
		
		if (StringMatch(HeaderWave[i], "*LIVETIME*"))
			HeaderWave[i] = "#LIVETIME  -s: " + num2str(LiveTime) +"\r\n"
		endif	
		
		if (StringMatch(HeaderWave[i], "*REALTIME*"))
			HeaderWave[i] = "#REALTIME  -s: " + num2str(RealTime) +"\r\n"
		endif	
		
		if (StringMatch(HeaderWave[i], "*DEADTIME*"))
			HeaderWave[i] = "#REALTIME  -s: " + num2str(DeadTime) +"\r\n"
		endif	
	endfor	
end


function iS_calcLiveTime(nPix, TimePerFrame, nFrames, DT, xDim, yDim)
	variable nPix, TimePerFrame, nFrames, DT, xDim, yDim
	
	variable TotTime = TimePerFrame * nFrames
	variable sPerPix = TotTime / (xDim * yDim)
	variable LiveTime = nPix * sPerPix * (1 - DT/100)
	return LiveTime
end



function iS_calcDT(areapc, LiveTime, TimePerFrame, nFrames,  xDim, yDim)  //calculates livetime from Compass results
	variable areapc, LiveTime, TimePerFrame, nFrames, xDim, yDim
	
	variable TotTime = TimePerFrame * nFrames
	variable sPerPix = TotTime / (xDim * yDim)
	variable nPix = (areapc/100) * xDim * yDim
	variable RealTime = nPix * sPerPix 
	variable DeadTime = 1- (LiveTime/RealTime)
	
	return DeadTime
end




// *************************************************************************************************
// --------------------------------- Display & Compare spetra functions--------------------------------------
// *************************************************************************************************


function iS_SetSpectrumGraph(graphName)
	string graphName
	
	doWindow $graphName
	
	if (V_flag)
		return 1
	endif
	
	Display /K=1  /W=(135,163,731,514) /N= $graphname
	
	ModifyGraph margin(top)=50
	//Label left "\\Z14 counts"
	Legend/C/N=text0/S=3
		
	PopupMenu AddSpectrum, Title = "Add", pos={20,5},size={50,20},  mode = 1,  popvalue="Select", value = iS_GetWaveListFromFolder(root:spectra:, "*"), proc = iS_DisplayWaveCtrl
	PopupMenu RemoveSpectrum, Title = "Remove", pos={180,5},size={50,20}, mode = 1,  popvalue="Select", value = TraceNameList("SpectrumDisplay", ";", 1), proc = iS_DisplayWaveCtrl
	Button syncColor,pos={380,5},size={80,20}, title="Sync Color", proc = iS_SyncDisplayColorsCtrl
	
	CheckBox normSpectra,pos={480,5},size={80,20}, title = "Normalise", proc = iS_NormaliseSpectraCtrl
	
	return 0
end



Function iS_SyncDisplayColorsCtrl(ctrlName) : ButtonControl				
	String ctrlName
	
	iS_SyncDisplayColors(root:Packages:iSpectra:M_Model, "SpectrumDisplay")	
End



Function iS_SyncDisplayColors(M_Model, graphName)		
	wave/T M_Model
	string GraphName
	
	if (!WaveExists(M_Model))
		DoAlert 0, "WaveExists() Error!"
	endif
	
	
	variable rr, gg, bb, i, group
	String Traces =TraceNameList(graphName, ";", 1)
	
	do		
		String Next = StringFromList(i, Traces)
		if (strlen(next) == 0)
			break
		endif
		
		// has trace on graph a default wave name, e.g. EDS_Group_3
		if (stringmatch(next, "EDS_Group_*") )

			sscanf next, "EDS_Group_%g", group  
			rr = str2num(M_Model[group][%R])
			gg = str2num(M_Model[group][%G])
			bb = str2num(M_Model[group][%B])	
			
			ModifyGraph rgb($next) = (rr, gg, bb)
			i+=1	
			continue
		
		// if not, trace is named after dimension label
		else
			
			group = FindDimLabel(M_Model, 0, next)
			if (group < 0 )
				break
			endif
			
			rr = str2num(M_Model[group][%R])
			gg = str2num(M_Model[group][%G])
			bb = str2num(M_Model[group][%B])	
			
			ModifyGraph rgb($next) = (rr, gg, bb)
			i+=1	
			continue
			
		endif	
			
		// if none of the other make it black

		ModifyGraph rgb($next) = (0, 0, 0)
		
		i+=1	
		while(1)	
End


function/S iS_getWaveListFromFolder(folder, sepString)
	DFREF Folder
	string SepString
	
	string dfsave = GetDataFolder(1)
	
	SetDataFolder Folder
	string List = WaveList(sepString, ";", "")
	SetDataFolder dfsave
	
	return list
end


Function iS_DisplayWaveCtrl(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	
	iS_DisplayWave(popstr, "SpectrumDisplay", ctrlname)
	
	return 1
End


Function iS_DisplayWave(wName, graphname, ctrlname) 
	String wName, graphname, ctrlname
		
	String Traces = TraceNameList(graphName, ";", 1)
	
	strswitch(ctrlName)	
		case "AddSpectrum":		
			
			wave w = $"root:Spectra:"+ wName
			
			if (strlen(Listmatch(traces, wName)) != 0)  // is popstr already in trace list?
				break
			endif
			
			AppendToGraph /W = $graphname w
			SetAxis /W = $graphname Bottom, 0, (MaxkeVSpecVal)
			break
									
		case "RemoveSpectrum":
			
			if (ItemsInlist(Traces) == 1)		// don't remove the last trace from graph
				break
			endif		
			
			RemoveFromGraph/Z /W = $graphName $wName
			break			
	endswitch
	
	return 1
End





// *************************************************************************************************
// --------------------------------- spectrum normalization      --------------------------------------
// *************************************************************************************************

Function iS_NormaliseSpectraCtrl(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked

	iS_NormaliseSpectra(root:spectra, checked)				
End


function iS_NormaliseSpectra(dfr, doUndo)  // doUndo: 1 for normalise; 0 for scaling back
	DFREF dfr
	variable doUndo
	
	String dfsave = Getdatafolder(1)
	SetDatafolder dfr
	
	String ListOfWaves = WaveList("*", ";", "")
	variable i
	
	do
		// run through spectra
		String Next = StringFromList(i, ListOfWaves)
		if (strlen(next) == 0)
			break
		endif
		wave w = $next
		
		// check wave note for status
		String wNote = Note(w)
		String Mode = StringByKey("MODE", wNote)
		variable nPix= str2num(StringByKey("PIXELS", wNote))
						
		Switch (doUndo)
	
			case 1:  	// normalise all waves and and change wave note to "scaled"
				
				// check if wave is already scaled
				if(cmpstr(mode, "scaled") == 0)
					break
				endif
				
				// normalise spectrum to one pixel
				w /= nPix
				
				// kill and re-write wave note
				Note/K w
				Note w, "MODE:scaled;PIXELS:" + num2str(nPix)
			break
		
			case 0:  //rescale to original values
				
				// check status
				if(cmpstr(mode, "full") == 0)
					break
				endif
				
				// multiply spectrum with number of pixels
				w *= nPix
				
				// kill and re-write wave note
				Note/K w
				Note w, "MODE:full;PIXELS:" + num2str(nPix)
			break
	
		EndSwitch
		
		i+=1
	while(1)
	
	SetDataFolder dfsave
end



// *************************************************************************************************
// --------------------------------- restore function     --------------------------------------
// *************************************************************************************************


function iS_PromptForRestorePoint()
	
	String RestoreName
	Prompt RestoreName, "Enter Name"
	DoPrompt "Enter Name For Restore Point", RestoreName
	
	if(V_flag)
		return 1
	endif
	
	RestoreName = CleanupName(RestoreName, 0)
	
	// call func to create restore waves
	iS_CreateRestorePoint(RestoreName) 

	return 0
end

function iS_CreateRestorePoint(RestoreName)
	string RestoreName
	
	wave/Z M_Groups = root:M_Groups
	DFREF dfr = root:packages:ispectra
	wave/Z M_Model = dfr:M_Model	
	wave/Z Settings = dfr:Settings
	wave/Z/T History = dfr:History
	
	if (!WaveExists(M_Groups) ||  !WaveExists(Settings)  ||  !WaveExists(M_Model)  ||  !WaveExists(History) )
		DoAlert 0, "WaveExists() error."
		return 0
	endif
	
	//if erosion is on, it NEEDS to be off! 
	if(Settings[%Erosion] == 1)
			DoAlert 0, "Turn off Erosion first! Then try again!"
			return 0
	 endif

	String restorePath = "root:Restore:"+RestoreName
	
	NewDataFolder/O root:Restore
	NewDataFolder/O $RestorePath
	
	Duplicate/O M_Groups $RestorePath+":M_Groups"
	Duplicate/O M_Model $RestorePath+":M_Model"
	Duplicate/O Settings $RestorePath+":Settings"
	
	// Write to History
	string cmd = ""
	sprintf cmd, "CMD:iS_CreateRestorePoint(\"%s\")", RestoreName
	iS_WriteString2History(cmd)

	return 0
end




function iS_LoadRestorePanel()
	if (!DataFolderExists("Root:Restore:") )
		return 0
	endif
	
	dowindow /F Restore
	if (V_flag)
		return 1
	endif

	NewPanel/N=Restore/K=1 /W=(1213,197,1539,271)
	
	PopupMenu SelectRestoreFolder,pos={18,24},size={140,20},title="Select restore point", proc = iS_ParseRestoreFolder
	PopupMenu SelectRestoreFolder,mode=1,popvalue="choose",value=  ReplaceString(",", StringByKey("FOLDERS", DataFolderDir(1, root:Restore)), ";")
EndMacro



Function iS_ParseRestoreFolder(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr

	string DFRstring = "root:Restore:" + popstr
	iS_LoadRestorePoint(DFRstring)
End


function iS_LoadRestorePoint(DFRstring)
	String DFRstring
	
	DFREF dref = $DFRstring
	
	wave/Z M_Model = dref:M_Model
	wave/Z M_Groups = dref:M_Groups
	wave/Z Settings = dref:Settings
	
	// Restore waves
	Duplicate/O M_Model root:packages:ispectra:M_model
	Duplicate/O Settings root:packages:ispectra:Settings
	Duplicate/O M_Groups root:M_Groups
	
	//Update panel
	iS_iSpectraGroupPanel()	
	iS_DisplayGroupsOnGrey(root:packages:ispectra:M_model, root:M_Groups, root:M_SumImage)
	
	// Write to History
	
	//wave/T/Z History =  root:packages:ispectra:History
	string cmd = ""
	sprintf cmd, "CMD:iS_LoadRestorePoint(\"%s\")", DFRstring
	iS_WriteString2History(cmd)
	
end


// *************************************************************************************************
// ---------------------------------  Save & Export function --------------------------------------
// *************************************************************************************************


function iS_FullSave() 

	DFREF dfr = root:packages:iSpectra
	SVAR/Z CubeFileName = dfr:CubeFileName
	
	//Where to save	
	NewPath/O /M="Select a location for the output folder" FolderPath
	if(V_Flag!=0)
		return NaN
	endif
	
	if (strlen(CubeFileName)==0)
		CubeFileName = "Cube"
	endif
	
	PathInfo FolderPath
	NewPath/Q /C/O SavePath, S_path+CubeFileName
	PathInfo SavePath	
	
	// Does HeaderWave exist	
	wave/T/Z header = root:EmsaHeader	
	if ( !WaveExists(header) )
		iS_EMSAHeaderLoad("")	
		wave/T/Z header = root:EmsaHeader	
	endif
	
	
	iS_FullExport(S_Path, 1)
end




function iS_FullExport(FolderPathString,HTMLSaveOnOff, [ name])
	string FolderPathString
	variable HTMLSaveOnOff
	string name
	
	string saveName=""
	NewPath/Q/C/O SavePath, FolderPathString
	
	//Make a Summary notebook
	DoWindow/F Summary
	if(!V_flag)
		NewNoteBook /N=Summary /F=1 /K=1
		NoteBook Summary, fSize = 20, fStyle = 1, text = "Export Summary\r\r"
	endif
	
	//Get Image dimensions for HTML scaling
	wave/Z M_Groups = root:M_Groups
	variable rows = DimSize(M_Groups, 0)
	variable scale = HTMLimageWidth/rows *100
	
	// save grey scale image	
	wave/Z M_SumImage = root:M_SumImage
	if (WaveExists(M_SumImage))
		
		if(ParamIsDefault(name))
			SaveName = "M_SumImage.tif"
		else
			saveName = "M_SumImage_"+name+".tif"
			
			//save to Notebook
			NoteBook Summary, fSize = 12, fStyle = 1, text = name +"\r\r"	
		endif
		
		//Save to Disk....
		ImageSave/O/IGOR/T="TIFF" /D=16 /P = SavePath M_SumImage as SaveName
		//and to Notebook
		NoteBook Summary, scaling={scale,scale}, insertpicture = {$"", savepath, saveName, 0}, text = "\r \r"	
	endif
	
	// save RGB Image if it exists
	wave/Z M_RGBImage = root:M_RGBImage
	if (WaveExists(M_RGBImage))
		
		if(ParamIsDefault(name))
			SaveName = "M_RGBImage.tif"
		else
			saveName = "M_RGBImage_"+name+".tif"
			
			//save to Notebook
			NoteBook Summary, fSize = 12, fStyle = 1, text = name +"\r\r"	
		endif
		
		//Save to Disk....
		ImageSave/O/IGOR/T="TIFF" /D=24 /P = SavePath M_RGBImage as SaveName
		//and to Notebook
		NoteBook Summary, scaling={scale,scale}, insertpicture = {$"", savepath, saveName, 0}, text = "\r \r"	
	endif
	
	
	// save groups image; first make a 3-layer TIFF
	DFREF dfr = root:packages:iSpectra
	wave/Z M_Groups = root:M_Groups
	wave/T/Z M_Model = dfr:M_Model
	
	// before exporting group panel make sure the pixel counts are up to date
	iS_UpdatePixelcount()
	
			
	if (WaveExists(M_Groups) && WaveExists(M_Model) )
		
		// export uneroded image
		iS_Erode(0)
	
		Duplicate/FREE/O M_Groups, M_tmp
		Redimension/D/N=(-1,-1, 3) M_tmp
		M_tmp = 0
		
		variable i
		for (i=0; i<DimSize(M_Model, 0); i+=1)
		
			variable red = str2num( M_Model[i][%R] )
			variable green = str2num(M_Model[i][%G] )
			variable blue = str2num( M_Model[i][%B] )
			
			MultiThread M_tmp[][][0] += M_Groups[p][q] == i ? red : 0
			MultiThread M_tmp[][][1] += M_Groups[p][q] == i ? green : 0
			MultiThread M_tmp[][][2] += M_Groups[p][q] == i ? blue : 0
		endfor
		
		if(ParamIsDefault(name))
			SaveName = "M_Groups.tif"
		else
			saveName = "M_Groups_"+name+".tif"
		endif
		
		ImageSave/O/IGOR/D=24/T="TIFF"/P=SavePath M_tmp as SaveName
		NoteBook Summary, scaling={scale,scale}, insertpicture = {$"", savepath, saveName, 0}, text = "\r \r"
		
		// export eroded image
		iS_Erode(1)
		M_tmp = 0
		
		for (i=0; i<DimSize(M_Model, 0); i+=1)
		
			red = str2num( M_Model[i][%R] )
			green = str2num(M_Model[i][%G] )
			blue = str2num( M_Model[i][%B] )
			
			MultiThread M_tmp[][][0] += M_Groups[p][q] == i ? red : 0
			MultiThread M_tmp[][][1] += M_Groups[p][q] == i ? green : 0
			MultiThread M_tmp[][][2] += M_Groups[p][q] == i ? blue : 0
		endfor
		
		if(ParamIsDefault(name))
			SaveName = "M_Groups_Eroded.tif"
		else
			saveName = "M_Groups_Eroded_"+name+".tif"
		endif
		
		ImageSave/O/IGOR/D=24/T="TIFF"/P=SavePath M_tmp as SaveName
		NoteBook Summary, scaling={scale,scale}, insertpicture = {$"", savepath, saveName, 0}, text = "\r \r"
			
	endif
	
	//now save the panel
	
	if(ParamIsDefault(name))
		SaveName = "GroupPanel.tif"
	else
		saveName = "GroupPanel_"+name+".tif"
	endif
	
	iS_UpdatePixelCount()   		//make sure all pixel values are up to date
	DoWindow/F iSpectraGroups
	iS_runParticleAnalysis()		// mark particles
	if (V_Flag)
		SavePict/O/WIN=iSpectraGroups/SNAP=1/P=SavePath/E=-7 as SaveName
		NoteBook Summary, insertpicture = {$"", savepath, saveName, 0}, text = "\r \r"
	else
		iS_iSpectraGroupPanel()
		SavePict/O/WIN=iSpectraGroups/SNAP=1/P=SavePath/E=-7 as SaveName
		NoteBook Summary,  insertpicture = {$"", savepath, saveName, 0}, text = "\r \r"
		KillWindow iSpectraGroups
	endif
	
	NoteBook Summary, fSize = 12, fStyle = 0, text = "----------------------------------------------------------\r \r"		
	
	// save panel only in single export - not in batch process
	if(HTMLSaveOnOff)
		DoWindow/F Summary
		if (V_flag)
			SaveNoteBook /S=5 /P= savePath Summary as "Summary.htm"
			DoWindow/K Summary
		endif
	endif
	
	//now save all spectra
	
	if(ParamIsDefault(name))
		SaveName = ""
	else
		saveName = name
	endif
	iS_ExportSpectra(1, folderpath = FolderPathString, name = saveName)		
	
	// turn erosion OFF, e.g. Settings[%Erosion] = 0; causes otherwise problem in batch processing and reestore points
	iS_Erode(0)			
end




// *************************************************************************************************
// ---------------------------------  Batch processing --------------------------------------
// *************************************************************************************************


function iS_SetupBatchProcessing()

	wave/T/Z BatchSetup = root:BatchSetup
	
	if(!WaveExists(BatchSetup))
		Make/T/O/N=8 BatchSetup
		SetDimLabel 0, 0, History, BatchSetup
		SetDimLabel 0, 1, EmsaHeader, BatchSetup
		SetDimLabel 0, 2, ListOfCubes, BatchSetup
		SetDimLabel 0, 3, PixPcLimit, BatchSetup
		SetDimLabel 0, 4, mode, BatchSetup
		SetDimLabel 0, 5, RGB, BatchSetup
		SetDimLabel 0, 6, BatchName, BatchSetup
		SetDimLabel 0, 7, BatchSavePath, BatchSetup		
	else
		DoAlert 2, "A BatchSetup wave already exists?\rDo you want to overwrite (YES) or run it (NO)?"
		if(V_flag == 1)
			//just go on after if-endif
		endif
		if(V_flag == 2)
			iS_runBatch(BatchSetup)
			return 2
		endif
		if(V_flag == 3)
			return 3
		endif
	endif
		
	variable RefNum
	
	// Get history wave
	Open/D/R/F="*.itx"/M="Select History Wave" refNum
	if (strlen(S_FileName) == 0) // user cancelled or some error occured
		DoAlert 0, "User cancelled!"
		return -1
	endif
	BatchSetup[%History] = S_fileName
	
	// get spectrum header wave
	Open/D/R/F="*.emsa"/M="Select an EMSA spectrum" refNum
	if (strlen(S_FileName) == 0) // user cancelled or some error occured
		DoAlert 0, "User cancelled!"
		return -1
	endif
	BatchSetup[%EmsaHeader] = S_fileName
	
	// next: get a List of SI raw files to be processed
	BatchSetup[%ListOfCubes] =  iS_GetOneRPLFilePath(1)
	if (strlen(BatchSetup[%ListOfCubes]) == 0) // user cancelled or some error occured
		DoAlert 0, "Nothing to process!"
		return -1
	endif
		
	//next: ask what to do with un-named group
	variable PixPcLimit
	variable mode
	
	DoAlert 1, "Do you want to process un-named groups; groups with no entry in column \"Phase\")"
	if (V_flag==1) //yes clicked
		Prompt PixPcLimit, "Enter pixel% under which un-named groups will be merged with TRASH"
		Prompt mode, "Enter mode: 1 for additional particle analysis, 0 for no particle analysis"
		DoPrompt "Enter parameters", PixPcLimit, mode
		if(V_flag)
			return -1
		endif
	endif
	
	BatchSetup[%PixPcLimit] = num2str(PixPcLimit) 
	BatchSetup[%mode] = num2str(mode)
	
	//next: Export RGB images
	string RGB
	Prompt RGB, "Export RGB Image? Enter elements as list or leave blank for no export."
	DoPrompt "Enter Elements", RGB
	if(V_flag)
		return -1
	endif
	BatchSetup[%RGB] = RGB
	
	
	//next: ask for a base name for batch full export
	String BatchName
	Prompt BatchName, "Enter a name for the output folder"
	DoPrompt "Enter name", BatchName
	if(V_flag)
		return -1
	endif
	BatchSetup[%BatchName] = BatchName
	
	
	// finally ask for the outputpath, or does it exist
	NewPath/Q/O/M="Select the output directory" BatchSavePath
	if(V_Flag!=0)	//user cancelled
		return -1
	endif
	PathInfo BatchSavePath // update string S_Path
	BatchSetup[%BatchSavePath] = S_Path
	
	DoAlert 1, "Batch processing is set up. Start now?"
	if(V_flag == 1)
		 iS_runBatch(BatchSetup)
	endif
	
	return 0
end


function iS_runBatchFromMenu()
	wave/Z/T BatchSetup
	
	if(!WaveExists(BatchSetup))
		DoAlert 0, "WaveExists() error"
		return 0
	endif
	
	iS_runBatch(BatchSetup)
end



Function iS_runBatch(BatchSetup)
	wave/Z/T BatchSetup
	
	if(!WaveExists(BatchSetup))
		DoAlert 0, "WaveExists() error"
		return 0
	endif
	
	//1) Load History wave
	String FileNameString = BatchSetup[%History] 
	if(strlen(FileNameString)==0)
		DoAlert 0, "No History wave!\rBatch processing will terminate!"
		return 0
	endif
	LoadWave/Q/T /O FileNameString
	Wave/T/Z History = root:History
	
	//2) Load header
	if(strlen(BatchSetup[%EmsaHeader])==0)
		DoAlert 0, "No EMSA header!\rBatch processing will terminate!"
		return 0
	endif	
	iS_EmsaHeaderLoad(BatchSetup[%EmsaHeader])
	Wave/T Header = root:EmsaHeader
	
	//3) Get List of Cubes
	String ListOfCubes = BatchSetup[%ListOfCubes]
	if(strlen(ListOfCubes) == 0)
		DoAlert 0, "No SI's to process!\rBatch processing will terminate!"
		return -1
	endif
	variable numfiles = ItemsInList(ListOfCubes, "\r")
	
	//4) get names
	string BatchName = BatchSetup[%BatchName]
	if(strlen(BatchName) == 0)
		DoAlert 0, "No folders name specified!\rBatch processing will terminate!"
		return -1
	endif
	
	//5) get Path
	string BatchSavePath = BatchSetup[%BatchSavePath]
	if(strlen(BatchSavePath) == 0)
		DoAlert 0, "No output path specified!\rBatch processing will terminate!"
		return -1
	endif
	
	//6) get parameters for un-assigned pixels
	variable PixPcLimit = str2num(BatchSetup[%PixPcLimit])
	variable mode = str2num(BatchSetup[%mode])
	
	//7) get elements for RGB image
	
	string RGB = BatchSetup[%RGB]
	
		
	// create a batch summary wave which contains all M_Models
	Make/T/O/N=(0,10) BatchSummary
	wave/T BatchSummary = root:BatchSummary
	string labels = "Group;R;G;B;List;Pixels;PixPercent;EDSPixels;Phase;Show;"
	iS_WaveLabel(BatchSummary, labels, 1)			//label wave
	 
	 variable timer = startMStimer
	 
	 // make logfile
	variable HistoryStart = CaptureHistoryStart()
	NewNoteBook /N=logfile /F=0 /K=1
		
	printf "   \r"	
	Print "************ Batch processing starts ************ \r"
	printf "   \r"
	
	variable i=0
	do
		
		string ThisCube = StringFromList(i, ListOfCubes, "\r")
		if(strlen(ThisCube) == 0)
			break
		endif
		
		printf "   \r"
		printf "--------- Now proceccing dataset    %g out of %g    , %s  ---------\r", (i+1), numfiles, ThisCube
		
		
		// call batch process 
		iS_DoBatchProcess(History, Header, BatchSummary, ThisCube, BatchName, BatchSavePath, i, PixPcLimit, mode, RGB)
		
		i+=1 
				
	while(1)
	
	// Save BatchSummaryWave 
	Save/O/U={0,0,2,0} /J/DSYM=","/W /P = SavePath  :BatchSummary as "BatchSummary.txt"
	
	
	//KillWaves/Z root:BatchSummary
	
	printf "   \r"
	Printf "Total time for batch processing: %g minutes.", StopMStimer(timer)/1e6/60
	printf "   \r"
	Print "************  Batch processing finished ************ \r"
	printf "   \r"
	
	// capture log file information
	String text = CaptureHistory(HistoryStart, 0)
	NoteBook logfile, text = text
	SaveNoteBook/O/P= savePath logfile as "logfile.txt"
	DoWindow/K logfile
	
	//export HTML output
	DoWindow/F Summary
	if (V_flag)
		SaveNoteBook/O/S=5 /P= savePath Summary as "Summary.htm"
		DoWindow/K Summary
	endif 
	return 1
end



function iS_DoBatchProcess(History, Header, BatchSummary, RPLFilePathString, BatchName, OutputPathString, batchNum, PixPcLimit, mode, RGB)
	wave History, Header
	wave/T BatchSummary
	string RPLFilePathString, BatchName, OutputPathString
	variable batchNum
	variable PixPcLimit, mode
	string RGB
	
	
	// new Entry in BatchSummary
	
	String FileNameSuffix = ""
	variable SuffixStart = strsearch(RPLFilePathString, "[", 0)
	if( SuffixStart == -1)
		FileNameSuffix = BatchName+"_" + num2str(batchNum)
	else 
		variable SuffixEnd = strsearch(RPLFilePathString, "]", suffixStart)	
		FileNameSuffix = RPLFilePathString[SuffixStart, suffixEnd]
	endif
	
	variable pnts = DimSize(BatchSummary,0)
	InsertPoints pnts, 1, BatchSummary
	BatchSummary[pnts][0]=FileNameSuffix
	
	// load file
	iS_ReadLispixFilePair(RPLFilePathString)
		
	// apply History Wave
	iS_ApplyHistoryWave(History)
	
	//deal with un-named groups if desired
	if (PixPcLimit != 0)
		iS_MergeUnNamedGroups(PixPcLimit, mode)
	endif
	
	//Generate RGB Image if desired
	if(strlen(RGB) != 0)
		string dfsave = GetDataFolder(1)
		
		string red = StringFromList(0, RGB, ";")
		string green = StringFromList(1, RGB, ";")
		string blue = StringFromList(2, RGB, ";")
		
		SetDataFolder root:Intensities:
		iS_RGBGenerator(red, green, blue)
		wave M_RGBImage
		iS_ScaleIfMagIsKnown(M_RGBImage)
	
		Duplicate/O M_RGBImage root:M_RGBImage
		KillWaves/Z M_RGBImage
		SetDataFolder $dfsave 
	endif

	
	// do a full export
	string pathString = OutPutPathString+BatchName
	
	iS_FullExport(pathString, 0, name = FileNameSuffix)
	
	
	// Add M_Model to BatchSummary wave
	wave/Z/T M_Model = root:packages:ispectra:M_Model
	//Concatenate/KILL/O/NP=0 {BatchSummary, M_Model}, Dest

	Concatenate/O/NP=0 {BatchSummary, M_Model}, Dest
	KillWaves/Z BatchSummary
	Rename dest BatchSummary
	
	//clean up
	Killwaves/Z :cube, :M_SumImage, :M_GroupsErodeUndo, M_RGBImage
	KillWaves/Z :M_Groups
	KillDataFolder/Z root:Packages
	KillDataFolder/Z root:Intensities
	KillDataFolder/Z root:Restore	
	
end




function iS_MakeReportFromBatchSummary(BatchSummary)
	wave/Z/T BatchSummary
	
	if (!WaveExists(BatchSummary))
		DoAlert 0, "WaveExists Error!"
		return 0
	endif
	
	variable nPnts = DimSize(BatchSummary, 0)
	
	Make/FREE/T/O/N=(nPnts) Phases = BatchSummary[p][%Phase]
	Make/FREE/O/N=(nPnts) Pixels = str2num(BatchSummary[p][%Pixels])
	
	variable i
	string AllPhases = ""
	String red="", green="", blue=""
	
	for (i=0; i< npnts; i+=1)
	
		string NextPhase = Phases[i]
		
		string redStr = BatchSummary[i][%R]
		string greenStr = BatchSummary[i][%G]
		string blueStr = BatchSummary[i][%B]
		
		if(FindListItem(NextPhase, AllPhases) == -1 && strlen(NextPhase) != 0)
			AllPhases += NextPhase +";"
			red += redstr +";"
			green += greenstr +";"
			blue += bluestr +";"
		endif	
	
	endfor	
	
		
	Make/FREE/T/N=(ItemsInList(AllPhases))/O, ContainedPhases= StringFromList(p, AllPhases)//, wRed = StringFromList(p,red), wGreen =StringFromList(p,green), wBlue=StringFromList(p,blue)
	Make/FREE/N=(ItemsInList(AllPhases))/O, TotalPixels = 0
	Make/FREE/N=(ItemsInList(AllPhases))/O, wRed = str2num(StringFromList(p,red)), wGreen =str2num(StringFromList(p,green)), wBlue=str2num(StringFromList(p,blue))
		
		
	for (i=0; i< npnts; i+=1)
	
		NextPhase = Phases[i]
		variable loc = WhichListItem(NextPhase, AllPhases)
		
		if(loc !=-1)
			TotalPixels[loc] += Pixels[i]
		endif		
	endfor	
	
	//Normalse TotalPix to % values
	variable tot = Sum(TotalPixels)
	Duplicate/O, TotalPixels, PixelPercent
	PixelPercent = TotalPixels[p]/tot * 100	
	
		
	// Make Legend
	nPnts = numPnts(ContainedPhases)
	variable xincr = 20, yincr = 15
	
	
	Display/K=1/N= PhaseLegend /W=(250,250,500,100 + 1.75*yincr*nPnts)
	SetDrawEnv xcoord= abs,ycoord= abs, fSize = 14
	DrawText 20,50, "Legend - Area %"
	

	for (i=0; i<npnts; i+=1)
		
		string Phase = ContainedPhases[i]
		string percentage 
		sprintf percentage, "%.2f", PixelPercent[i]
		variable varred = wRed[i], vargreen = wGreen[i], varblue = wBlue[i]
				
		SetDrawEnv xcoord= abs,ycoord= abs,linethick= 1,fillfgc= (varred,vargreen,varblue),fillbgc= (0,0,0)
		DrawRect 20, 70+yincr*i, 40, 80+yincr*i
		
		SetDrawLayer UserFront
		SetDrawEnv xcoord= abs,ycoord= abs, fSize = 10
		DrawText 50,80+yincr*i, Phase
		
		SetDrawLayer UserFront
		SetDrawEnv xcoord= abs,ycoord= abs, fSize = 10
		DrawText 150,80+yincr*i, percentage
	
	endfor
end




function iS_WriteSummaryLegend(Phases, PixPercent, R, G, B)
	wave/T Phases
	wave PixPercent, R, G, B
	
	variable nPnts = numPnts(Phases)
	variable i
	
	variable xincr = 20, yincr = 15
	
	Display/K=1/N= PhaseLegend /W=(250,250,500,100 + 1.75*yincr*nPnts)
	SetDrawEnv xcoord= abs,ycoord= abs, fSize = 14
	DrawText 20,50, "Legend - Area %"
	

	for (i=0; i<npnts; i+=1)
		
		string Phase = Phases[i]
		string percentage 
		sprintf percentage, "%.2f pc.", PixPercent[i]
		variable red = R[i], green = G[i], blue = B[i]
				
		SetDrawEnv xcoord= abs,ycoord= abs,linethick= 1,fillfgc= (red,green,blue),fillbgc= (0,0,0)
		DrawRect 20, 70+yincr*i, 40, 80+yincr*i
		
		SetDrawLayer UserFront
		SetDrawEnv xcoord= abs,ycoord= abs, fSize = 10
		DrawText 50,80+yincr*i, Phase
		
		SetDrawLayer UserFront
		SetDrawEnv xcoord= abs,ycoord= abs, fSize = 10
		DrawText 150,80+yincr*i, percentage
	
	endfor
end














// *************************************************************************************************
// --------------------------------- image stitching functions     --------------------------------------
// *************************************************************************************************

//function iS_LoadImagesForStitching()
//
//	DoAlert 1, "A Montage can only be created from SI's in grid or line mode, otherwise it will result in an error. Continue?"
//	if(V_flag == 2)
//		return 2
//	endif
//	
//		
//	// load image in temp folder
//	variable refNum		
//	String FileFilters = "IMAGE Files (*.tif,*.tiff,):.tif,.tiff;"
//	Open/F=FileFilters/D/R/M="Select Grid Files"/MULT=(1) refNum
//	string filePaths = S_fileName
//	if( strlen(filePaths) == 0 )
//		return 0	
//	endif
//	
//	string dfsave = GetDataFolder(1)
//	NewDataFolder/O/S ImageFolder
//			
//	variable i
//	do
//		
//		string ThisFile = StringFromList(i, filepaths, "\r")
//		if(strlen(Thisfile) == 0)
//			break
//		endif
//		
//		NewPath/Q/O StitchPath, ParseFilePath(1, ThisFile, ":", 1, 0)
//		string fileName = ParseFilePath(0, ThisFile, ":", 1, 0)
//		
//		ImageLoad/Q/T=tif /P= StitchPath fileName
//
//	i+=1	
//	while(1)
//	
//	SetDataFolder dfsave	
//	
//		
//	//prompt for x- and y-overlap
//	variable xol, yol
//	Prompt xol, "Enter vertical overlap in pixels: "
//	Prompt yol, "Enter horizontal overlap in pixels: "
//	DoPrompt "Enter pixel overlap", xol, yol
//	if(V_flag)
//		return -1
//	endif
//	
//	// write cmd to history for cpoy and pastw purposes
//	printf "Stitch(:ImageFolder, %g, %g) \r", xol, yol
//
//	iS_Stitch(:ImageFolder, xol, yol)	
//end
//	
//	
//	
//function iS_Stitch(dfr, xol, yol)
//	DFREF dfr				// where are the images?
//	variable xol, yol		// x- and y- overlap in pixels
//	
//	string dfsave = GetDataFolder(1)
//	SetdataFolder dfr
//	
//	// make a StitchWave which contains the image names in correct orientation
//	string List = WaveList("*", ";", "")
//	Make/FREE/T/O/N=(100,100) StitchWave
//	
//	variable rowCount=0, colCount=0
//	variable i=0
//	do
//		
//		String ThisFile = StringFromList(i, List, ";")
//		if(strlen(Thisfile) == 0)
//			break
//		endif
//		
//		variable startstr = strsearch(ThisFile, "@", 0)
//		variable endstr =  strsearch(ThisFile, "]", startStr)
//		
//		// now we got the numeric part of e.g. [Grid@1 1]
//		ThisFile = ThisFile[startStr+1, endStr-1]
//
//		//where is the white space?
//		variable space = strsearch(ThisFile, " ", 0)
//		
//		// get now the grid position
//		variable ycoord = str2num(ThisFile[0, space-1])
//		variable xcoord = str2num(ThisFile[space+1, strlen(ThisFile)])
//		
//		//test if there are valid coordinates
//		if(numtype(ycoord)==2 || numtype(xcoord) == 2)
//			print ThisFile
//			SetDataFolder dfsave
//			Abort "At least one invalid name!"
//		endif
//		
//		if(xcoord > rowCount)
//			rowCount +=1
//		endif
//		if(ycoord > colCount)
//			colcount+=1
//		endif
//				
//		StitchWave[xcoord][ycoord] = StringFromList(i, List, ";")
//		
//		i+=1	
//	while(1)	
//	
//	rowcount = rowcount == 0 ? 0 : rowcount+1
//	colcount = colcount == 0 ? 0 : colcount +1
//	ReDimension/N=(rowcount , colCount) StitchWave
//	
////	variable rows = DimSize(StitchWave, 0)
////	variable cols = DimSize(StitchWave, 1)
//		
//	variable j=0
//	i=0
//	do	
//		//get wavelist for one column
//		string rowList=""
//		
//		do
//		
//		
//			Duplicate/O $StitchWave[i][j] $"M_Image_"+num2str(i)+"_"+num2str(j)
//			wave tmp = $"M_Image_"+num2str(i)+"_"+num2str(j)
//			
//			// cut out overlapping pixels part, but NOT the last one
//			if (i!=rowCount-1)
//				Deletepoints /M=1 0, xol, tmp
//			endif
//		
//			rowList = AddListItem("M_Image_"+num2str(i)+"_"+num2str(j), rowList)
//		
//			i+=1
//		while(i<rowCount-1)
//		
//		//create Image of one row
//		Concatenate/KILL/O/NP=1 rowList, $"RowImage_Row_"+num2str(j)
//		wave StitchedRow = $"RowImage_Row_"+num2str(j)
//		
//		// trim row image for overlapped part, but NOT the last one
//		if(j!=colCount-1)
//			DeletePoints/M=0 (DimSize(StitchedRow, 0)-yol), yol, StitchedRow 
//		endif
//	
//		j+=1
//	while(j<colCount)
//		
//	string ListOfRows = WaveList("RowImage*", ";", "")
//	Concatenate /KILL/O /NP=0 ListOfRows, root:M_ImageMontage
//	
//	
//	DoWindow/F Montage
//	if(!V_flag)
//		NewImage/K=1 /N=Montage root:M_ImageMontage
//	endif	
//	
//		
//	SetDataFolder dfsave
//	
//end

function iS_LoadImagesForStitching()

	DoAlert 1, "A Montage can only be created from SI's in grid or line mode, otherwise it will result in an error. Continue?"
	if(V_flag == 2)
		return 2
	endif
	
		
	// load image in temp folder
	variable refNum		
	String FileFilters = "IMAGE Files (*.tif,*.tiff,):.tif,.tiff;"
	Open/F=FileFilters/D/R/M="Select Grid Files"/MULT=(1) refNum
	string filePaths = S_fileName
	if( strlen(filePaths) == 0 )
		return 0	
	endif
	
	string dfsave = GetDataFolder(1)
	NewDataFolder/O/S ImageFolder
			
	variable i
	do
		
		string ThisFile = StringFromList(i, filepaths, "\r")
		if(strlen(Thisfile) == 0)
			break
		endif
		
		NewPath/Q/O StitchPath, ParseFilePath(1, ThisFile, ":", 1, 0)
		string fileName = ParseFilePath(0, ThisFile, ":", 1, 0)
		
		ImageLoad/Q/T=tif /P= StitchPath fileName

	i+=1	
	while(1)
	
	SetDataFolder dfsave	
	
		
	//prompt for x- and y-overlap
	variable xol, yol
	Prompt xol, "Enter vertical overlap in pixels: "
	Prompt yol, "Enter horizontal overlap in pixels: "
	DoPrompt "Enter pixel overlap", xol, yol
	if(V_flag)
		return -1
	endif
	
	// write cmd to history for cpoy and pastw purposes
	printf "iS_Stitch(:ImageFolder, %g, %g) \r", xol, yol

	iS_Stitch(:ImageFolder, xol, yol)	
end
	
	
	
function iS_Stitch(dfr, xol, yol)
	DFREF dfr				// where are the images?
	variable xol, yol		// x- and y- overlap in pixels
	
	string dfsave = GetDataFolder(1)
	SetdataFolder dfr
	
	// make a StitchWave
	string List = WaveList("*", ";", "")
	Make/FREE/T/O/N=(100,100) StitchWave
	
	variable maxX=1, MaxY=1
	variable i=0
	do
		
		String ThisFile = StringFromList(i, List, ";")
		if(strlen(Thisfile) == 0)
			break
		endif
		
		variable startstr = strsearch(ThisFile, "@", 0)
		variable endstr =  strsearch(ThisFile, "]", startStr)
		
		// now we got the numeric part of e.g. [Grid@1 1]
		ThisFile = ThisFile[startStr+1, endStr-1]

		//where is the white space?
		variable space = strsearch(ThisFile, " ", 0)
		
		// get now the grid position
		variable xcoord = str2num(ThisFile[0, space-1])
		variable ycoord = str2num(ThisFile[space+1, strlen(ThisFile)])
		
		if (xcoord > MaxX)
			MaxX=xcoord
		endif
		if (ycoord > MaxY)
			MaxY=Ycoord
		endif	
		
		StitchWave[ycoord][xcoord] = StringFromList(i, List, ";")
		
		i+=1	
	while(1)	
	
	ReDimension/N=(maxY+1, maxX+1) StitchWave
	
	
	variable rows = DimSize(StitchWave, 0)
	variable cols = DimSize(StitchWave, 1)
		
	variable j=0
	for(j=0; j<cols; j+=1)
		
		//get wavelist for one column
		string rowList=""
		//variable i
		for(i=0; i<rows; i+=1)
		
			Duplicate/O $StitchWave[i][j] $"M_Image_"+num2str(i)+"_"+num2str(j)
			wave tmp = $"M_Image_"+num2str(i)+"_"+num2str(j)
			
			// cut out overlapped part, but NOT the last one
			if (i!=rows-1)
				Deletepoints /M=1 0, xol, tmp
			endif
		
			rowList = AddListItem("M_Image_"+num2str(i)+"_"+num2str(j), rowList)
		endfor

		//create Image of one row
		Concatenate/KILL/O/NP=1 rowList, $"RowImage_Row_"+num2str(j)
		wave StitchedRow = $"RowImage_Row_"+num2str(j)
		
		// trim row image for overlapped part, but NOT the last one
		if(j!=cols-1)
			DeletePoints/M=0 (DimSize(StitchedRow, 0)-yol), yol, StitchedRow 
		endif
	
	endfor
		
	string ListOfRows = WaveList("RowImage*", ";", "")
	Concatenate /KILL/O /NP=0 ListOfRows, root:M_ImageMontage
	
	
	DoWindow/F Montage
	if(!V_flag)
		NewImage/K=1 /N=Montage root:M_ImageMontage
	endif	
	
		
	SetDataFolder dfsave
	
end



// *************************************************************************************************
// ---------------------------------  Single Phase Analysis        --------------------------------------
// *************************************************************************************************


Function iS_PromptForSinglePhaseAnalysis()
	Variable group, minPixels, circularity
	
	wave/Z M_Groups = root:M_Groups
	wave/Z cube = root:cube
	
	if (!WaveExists(M_Groups) || !WaveExists(Cube))
		DoAlert 0, "WaveExists() error."
		return 0
	endif
	
	Prompt group, "Enter group number: "		
	Prompt minPixels, "Minimum pixels per particle"		
	DoPrompt "Enter search parameters", group, minPixels
	if (V_Flag)
		return -1								
	endif
	
	// ask for export
	variable export = 0
	string S_Path = ""
	
	DoAlert 1, "Export Spectra?"
	if(V_flag ==1)	// yes clicked
		export = 1
		wave/Z/T EmsaHeader = root:EmsaHeader
		if(!WaveExists(EmsaHeader))
			iS_EmsaHeaderLoad("")
		endif
		
		PathInfo SpectrumPath	// does an export path exist?
		if(!V_flag)				//no
			NewPath/O/M="Select output folder on disc" SpectrumPath
			Pathinfo SpectrumPath	
		endif	
		
	endif
	print S_Path	
	iS_SinglePhaseAnalysis(group, minPixels, export, S_Path) 	
End


function iS_SinglePhaseAnalysisAllPhases(minPixels)
	variable minPixels
	
	wave/Z M_Model = :packages:iSpectra:M_Model
	Make/O/N=(Dimsize(M_Model,0)) NumParticlesPerGroups = p
	
	PathInfo SpectrumPath
	if(V_flag==0)
		NewPath/O/M="Select output folder on disc" SpectrumPath
		Pathinfo SpectrumPath	
	endif		
	
	NumParticlesPerGroups = iS_SinglePhaseAnalysis(p, minPixels, 1, s_Path) 
end


function iS_SinglePhaseAnalysis(group, minPixels, export, SpectrumPathOnDisc) 
	variable group, minPixels,  export
	string SpectrumPathOnDisc
	
	wave/Z M_Groups = root:M_Groups
	wave/Z cube = root:cube
	
	//check is spectrum folder exists
	if (!DataFolderExists(":Spectra:"))
		NewDataFolder root:Spectra
	endif
	
	string dfsave = GetDataFolder(1)
	NewDataFolder/O/S ParticleAnalysis
	
	// make binary wave for ImageAlalyseParticles
	Duplicate/O M_Groups, M_Group
	MultiThread M_Group = M_Group == group ? 0 : 255
	Redimension/B/U M_Group
	
	ImageAnalyzeParticles/Q /A=(minPixels) stats M_Group
	wave/Z W_SpotX, W_SpotY, W_circularity
	
	Make/O/B/U/N=(DimSize(cube, 0), DimSize(cube, 1)) M_ParticleMask=0
	iS_ScaleIfMagIsKnown(M_ParticleMask)
	
	variable i, numParticles=0
	for(i=0; i<numPnts(W_SpotX); i+=1)
		
		ImageAnalyzeParticles /L=(W_SpotX[i], W_SpotY[i]) mark M_Group
		wave/Z M_ParticleMarker
		
		//make mask for extracting spectra
		MultiThread M_ParticleMarker = M_ParticleMarker == 64 ? 0 : 1
		MultiThread M_ParticleMask += M_ParticleMarker
		
		// Make EDS spectrum
		Make/O/N=(DimSize(cube, 2)) $"root:Spectra:Group_" + num2str(group) +"_X_"+num2str(W_SpotX[i]) + "_Y_" +num2str(W_SpotY[i])
		wave EDS = $"root:Spectra:Group_" + num2str(group) +"_X_"+num2str(W_SpotX[i]) + "_Y_" +num2str(W_SpotY[i])
		MultiThread EDS = iS_WorkerFunc(cube, M_ParticleMarker, p)	
		SetScale/P x, 0, (0.01), "keV" EDS
		
		// add a wave note with number of pixels
		variable nPix = sum(M_ParticleMarker)
		Note EDS, "MODE:full;PIXELS:" + num2str(nPix)
		
		
		//is export to disc active
		if (export ==1)
			iS_ExportSpectrum( root:EmsaHeader, EDS, SpectrumPathOnDisc, "")
		endif
		
		numParticles +=1
	endfor
	
	DoWindow/F ParticleMask
	if (!V_flag)
		NewImage /K=1 /N=ParticleMask M_ParticleMask
	endif
	
	KillWaves/Z  W_ImageObjArea,W_circularity,W_rectangularity,W_ImageObjPerimeter,W_xmin,W_xmax,W_ymin,W_ymax,M_RawMoments,M_ParticleMarker  //W_SpotX,W_SpotY
	setDataFolder dfsave
	
	if (export ==1)
		Printf "%g Particle(s) exported to:    %s \r", numParticles, SpectrumPathOnDisc
	endif
	
	return numParticles
end

function iS_ShowSinglePhaseElement(Elem)
	string elem
	
	wave M_ParticleMask = root:ParticleAnalysis:M_ParticleMask
	wave M_Elem = $"root:Intensities:" + elem
	
	Duplicate/O M_Elem, $"root:ParticleAnalysis:"+elem
	wave dest = $"root:ParticleAnalysis:"+elem
	
	Dest = M_ParticleMask * M_Elem
end



function iS_FillParticles(M_ParticleMask, row, col, property)
	wave M_ParticleMask, row, col, property
	
	Make/O/D/N=(DimSize(M_ParticleMask,0), DimSize(M_ParticleMask,1)) M_Filled = 0
	variable i, np = Numpnts(property)	
	
	Redimension/D M_ParticleMask
	
	for (i=0;i<np;i+=1)
		ImageSeedfill/B=0 seedP = (row[i]), seedQ = (col[i]), target = property[i], srcWave = M_ParticleMask
		wave M_SeedFill	
		Multithread M_Filled += M_SeedFill	
	endfor	
		
	Multithread M_Filled = M_Filled ==0 ? NaN : M_Filled
	KillWaves/Z M_SeedFill
end


// *************************************************************************************************
// ---------------------------------  Correct phase Morpgology        --------------------------------------
// *************************************************************************************************


Function iS_GroupForImageMorphology() 	// set up panel
	Wave/Z/T M_Model = root:Packages:iSpectra:M_Model
	
	if(!WaveExists(M_Model))
		DoAlert 0, "WaveExists() error!"
		return 0
	endif
	
	
	Dowindow /F MorphologyPanel
	if (V_flag)
		return 1
	endif

	NewPanel/N=MorphologyPanel/K=1 /W=(800,300,1200,480)
	
	PopupMenu SelectMethod,pos={30,20},size={110,20},title="Select Method   ", mode=1,value=  "Erosion;Dilation;Opening (Erosion->Dilation);Closing (Dilation->Erosion);Opening-Closing;Closing-Opening;"
	PopupMenu SelectStructureElement,pos={30,50},size={110,20}, title="Select Structure Element   ", mode=1, value ="3x3 cross;5x5 circle;3x3 square;"
	PopupMenu SelectGroup,pos={30,80},size={110,20},title="Select group   ", mode=1,value=  iS_GetPhaseInformation()
	
	Button DoMorphology, pos={30,120},size={50,20},title="Do it", proc = iS_CallMorphologyFunction
	Button UndoMorphology, pos={100,120},size={50,20},title="Undo", proc = iS_CallMorphologyFunction
End


function/S iS_GetPhaseInformation()
	Wave/Z/T M_Model = root:Packages:iSpectra:M_Model
	
	if(!WaveExists(M_Model))
		DoAlert 0, "WaveExists() error!"
		return ""
	endif
	
	
	// create an appropriate item list string for popup
	variable i
	String Selection = ""
	
	for (i=0; i<DimSize(M_Model, 0); i+=1)
		Selection += "Group:" + num2Str(i) +"| " + M_Model[i][%Phase] + "| ELEMENTS:" + M_Model[i][%List] + "| ;" 
	endfor
	
	return selection		
end


Function iS_CallMorphologyFunction(ctrlName) : ButtonControl
	String ctrlName
	
	Wave/Z M_Groups = root:M_Groups
	if (!WaveExists(M_Groups))
		return 0
	endif
	
	String cmd = ""
	
	strswitch(ctrlname)	// string switch
		case "DoMorphology":	
			
			ControlInfo /W=MorphologyPanel SelectMethod; String Method = S_Value
			ControlInfo /W=MorphologyPanel SelectStructureElement; String sElement = S_Value
			ControlInfo /W=MorphologyPanel SelectGroup; String group = S_Value
			
			variable groupNum = NumberByKey("Group", group, ":")
			string groupStr = StringByKey(" ELEMENTS", group, ":", "|")

			iS_PhaseMorphology(M_Groups, groupStr, sElement, Method)
			break
			
		case "UndoMorphology":		
		
			Wave/Z M_undo = root:M_Groups_MorphologyUndo
			if (!WaveExists(M_Undo))
				return 0
			endif
			
							
			M_Groups = M_Undo
			
			// write to history
			cmd = "CMD:M_Groups = M_Groups_MorphologyUndo"
			iS_WriteString2History(cmd)
						
			KillWaves/Z M_Undo
			iS_UpdatePixelCount()
		
			break
			
		default:
			break	
			
	endswitch
End


function iS_PhaseMorphology(M_Groups, groupstr, StructureElement, WhatToDo)
	wave M_Groups
	String groupstr
	string StructureElement, WhatToDo
	
	variable strucElement
	
	Duplicate/FREE/O M_Groups, M_Temp
	Duplicate/O M_Groups, M_Groups_MorphologyUnDo  // make undo wave
	
	
	wave/T/Z M_Model = root:packages:ispectra:M_Model
	if(!WaveExists(M_Model))
		return 0
	endif
	
	
	// find groupnumber which matches groupstr
	variable i, group = -1
	for (i=0; i<DimSize(M_Model,0); i+=1)	
		if(cmpstr(groupstr, M_Model[i][%List])==0)
			group = i
			break
		endif
	endfor
	
	
	// if group is not present in M_Model then skip it here. This may occur during batch processing
	if (group==-1)
		return 0
	endif
		
	MultiThread M_temp = M_Groups == group ? 1 : 0
	
	Redimension/B/U M_Temp
	Duplicate/FREE/O M_Temp, M_Mask
	

	strswitch(StructureElement)	// string switch
		case "3x3 cross":		
			
			strucElement = 4
			break
			
		case "5x5 circle":		
			
			strucElement = 5
			break
			
		case "3x3 square":		
			
			strucElement = 6
			break
			
		default:
			strucElement = 6
			break	
			
	endswitch

	
	strswitch(WhatToDo)	// string switch
		case "Opening-Closing":		
			
			ImageMorphology/O/I=1 /E=(strucElement) Opening M_Temp
			ImageMorphology/O/I=1 /E=(strucElement) Closing M_Temp
			break
			
		case "Closing-Opening":		
			
			ImageMorphology/O/I=1 /E=(strucElement) Closing M_Temp
			ImageMorphology/O/I=1 /E=(strucElement) Opening M_Temp
			break
			
		case "Closing (Dilation->Erosion)":		
			
			ImageMorphology/O/I=1 /E=(strucElement) Closing M_Temp
			break
			
		case "Opening (Erosion->Dilation)":		
			
			ImageMorphology/O/I=1 /E=(strucElement) Opening M_Temp
			break	
			
		case "Erosion":		
			
			ImageMorphology/O/I=1 /E=(strucElement) Erosion M_Temp
			break	
		
		case "Dilation":		
			
			ImageMorphology/O/I=1 /E=(strucElement) Dilation M_Temp
			break		
			
		default:
			break	
			
	endswitch

	// re-assign pixels into groups
	MultiThread M_Mask = M_Temp ==1 ? 0 : M_Mask
	MultiThread M_Groups = M_Mask == 1 ? 0 : M_Groups
	MultiThread M_Groups = M_Temp == 1 ? group : M_Groups
	iS_UpdatePixelCount()	
	
	//write to histoty
	string cmd
	sprintf cmd, "CMD:iS_PhaseMorphology(M_Groups, \"%s\", \"%s\", \"%s\")", groupstr, StructureElement, WhatToDo
	iS_WriteString2History(cmd)	
end




function iS_ExportPhaseOnGreyscale(group, M_Image, M_Groups, M_Model) // M_Image must be scaled to 16 bit values
	variable group
	wave M_Image, M_Groups
	wave/T M_Model
	
	Duplicate/O M_Image M_tmp
	Redimension/N=(-1,-1,3) M_tmp
	
	MultiThread M_tmp[][][1] = M_tmp[p][q][0]
	MultiThread M_tmp[][][2] = M_tmp[p][q][0]
	
	variable red = str2num( M_Model[group][%R] )
	variable green = str2num(M_Model[group][%G] )
	variable blue = str2num( M_Model[group][%B] )
			
	MultiThread M_tmp[][][0] = M_Groups[p][q] == group ? red : M_tmp[p][q][0]
	MultiThread M_tmp[][][1] = M_Groups[p][q] == group ? green : M_tmp[p][q][1]
	MultiThread M_tmp[][][2] = M_Groups[p][q] == group ? blue : M_tmp[p][q][2]
	
	ImageSave/O/D=24/T="TIFF" M_tmp
end






function iS_makeScaleBar(width, height, Mag)
	variable width, height, mag  //width and height of original image

	
	// fit from Image width vs magnification on Jeol LX 6200 with Thermo Fisher NSS7
	variable ImageWidth = 10^(-0.999*log(Mag) + 5.093)
	variable size = ImageWidth / width
	
	//Make scale bar 10% of image height
	Make/W/U/O/N=(width, round(height/12)) ScaleBar
	
	variable ScaleHeight = DimSize(ScaleBar, 1)
	ScaleBar = 0
	
	NewImage/N=ScaleBar/K=1 ScaleBar
	ModifyImage ScaleBar,explicit=1,eval={0,0,0,0},eval={255,0,0,0}
	ModifyGraph margin = -1
	
	//define scale bar min- and max dimensions in terms of image size fractions
	variable MinLengthFraction = 0.13
	variable MaxLengthFraction = 0.28
	variable ScaleBarCentre = 0.8
	variable ScaleBarHeight = 0.5
	variable ScaleBarLineThick = ScaleHeight * 0.1
	
	//calculate scale bar length in microns for MinLength and MaxLength
	variable MinMicrons = MinLengthFraction* width * Size
	variable MaxMicrons = MaxLengthFraction* width  * Size
	
	//Make a "allowed micron values" wave
	  Make/FREE/O ScaleValues = {0.01,0.1,0.2,0.5,1,2,5,10,25,50,100,250,500,1000, 5000}
	
	//Find Position - give larger scale bars priority
	FindLevel/Q ScaleValues, MaxMicrons
	if (V_flag)
		DoAlert 0, "Invalid magnification"
		return 0
	endif
	
	variable ScaleMicrons = ScaleValues[trunc(V_levelX)]
	variable ScaleLengthFraction = ScaleMicrons / Size / width
	
	//draw scale bar
	SetDrawEnv linefgc= (65535,65535,65535),linethick= ScaleBarLineThick
	DrawLine (ScaleBarCentre - ScaleLengthFraction/2), ScaleBarHeight,(ScaleBarCentre + ScaleLengthFraction/2), ScaleBarHeight
	
	//write text
	SetDrawEnv textrgb= (65535,65535,65535),textxjust= 1,textyjust= 1,fstyle= 1, fname ="Arial", fsize = round(ScaleHeight*0.5)
	DrawText ScaleBarCentre, ScaleBarHeight+0.2 , num2Str(ScaleMicrons) + "  \\F'Symbol'm\\F'Arial'm"
	
	// write magnification
	variable textheight = ScaleBarheight//(height+ScaleHeight/2)/(height+ScaleHeight)
	SetDrawEnv textrgb= (65535,65535,65535),textxjust= 0,textyjust= 1,fstyle= 1, fname ="Arial", fsize = round(ScaleHeight*0.6)
	DrawText 0.05, TextHeight , "Mag. " + num2str(mag) + " x"
	
	return 1
end





// *************************************************************************************************
// --------------------------------- Color Data Base Functions        --------------------------------------
// *************************************************************************************************


function iS_MakeNewColourDB()
	Wave/Z/T M_ColourDataBase = root:M_ColourDataBase
	
	if(!WaveExists(M_ColourDataBase))
		Make/O/T/N=(1, 4) root:M_ColourDataBase
	endif
	
	iS_ColourDBPanel(root:M_ColourDataBase) //open panel
end


function iS_LoadOrShowColourDB()
	Wave/Z/T M_ColourDataBase = root:M_ColourDataBase
	

	if(!WaveExists(M_ColourDataBase))
		LoadWave/T
		if (V_Flag == 0)
			return 0
		endif
		Wave/Z/T M_ColourDataBase = root:M_ColourDataBase
	endif

	iS_ColourDBPanel(root:M_ColourDataBase) //open panel
	return 1
end


function iS_SaveColourDB()
	Wave/Z/T M_ColourDataBase = root:M_ColourDataBase
	
	if(!WaveExists(M_ColourDataBase))
		return 0
	endif

	Save/T M_ColourDataBase as "M_ColourDataBase.itx"
	return 1
end


Function iS_ColourDBPanel(M_ColourDB) : Panel
	wave/T M_ColourDB
	
	if(!WaveExists(M_ColourDB))
		return 0
	endif

	variable n = DimSize(M_ColourDB, 0)
	variable i, left, top
	variable y0= 20
	variable space = 22
	
	
	//Get the position of the panel if already open
	DoWindow ColourDBPanel
	if (V_flag == 1)    //Panel exists --> //Recreate panel at its last position
		GetWindow ColourDBPanel wsizeRM	
		top = V_top*ScreenResolution/72
		left = V_left * ScreenResolution/72				
		DoWindow /K ColourDBPanel
		NewPanel /N=ColourDBPanel /K=1 /W=(left,top,left+200,top+90+n*space)
		ModifyPanel/W=ColourDBPanel  noEdit=0//, fixedSize=1  // disable modification on its size
	else  //Panel does not exists --> create at default position
		left = 200
		top = 500
		NewPanel /N=ColourDBPanel /K=1 /W=(left,top,left+200,top+90+n*space)
	endif
	
	
	for (i=0; i<(n); i+=1)	
		PopupMenu $("ColourPicker_"+num2str(i)) ,pos={5,y0+i*space},size={50,20}, mode = 1, win = ColourDBPanel, popcolor = (str2num(M_ColourDB[i][0]) , str2num(M_ColourDB[i][1]),str2num(M_ColourDB[i][2])), value= #"\"*COLORPOP*\"", proc =iS_Colour2Wave
		SetVariable $("Phase_" + num2str(i)), title = " ", pos = {75, y0+2+(i*space)}, win = ColourDBPanel, size= {100, 50}, value = M_ColourDB[i][3], proc =  iS_AddColourDBNameCtrl 
	endfor
	
	Button SyncCDBButton, pos={45, y0+(i*space)+10},size={100,20}, title="Sync Color", proc = iS_SyncDBCtrl
	Button AddOne, pos={45, y0+(i*space)+35},size={100,20}, title="New Entry", proc = iS_NewEntryCtrl
end

Function iS_Colour2Wave(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	Variable r, g, b
		
	Wave/Z/T M_ColourDB = root:M_ColourDataBase
	
	if(!WaveExists(M_ColourDB))
		return 0
	endif
	
	variable group; 	sscanf ctrlname, "ColourPicker_%g", group  
	
	r= str2num(popstr[1,inf])
	variable spos= strsearch(popstr,",",0)
	g= str2num(popstr[spos+1,inf])
	spos= strsearch(popstr,",",spos+1)
	b= str2num(popstr[spos+1,inf])
	
	M_ColourDB[group][0] = num2str(r)
	M_ColourDB[group][1] = num2str(g)
	M_ColourDB[group][2] = num2str(b)
	
	return 1
End


Function iS_AddColourDBNameCtrl(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum	
	String varStr		
	String varName		
	
	varStr = CleanUpName(VarStr,0)
		
	wave/Z/T M_ColourDataBase = root:M_ColourDataBase
	variable entryNum;	sscanf ctrlname, "Phase_%g", entryNum  
	
	M_ColourDataBase[entryNum][3]	=  varStr 
End



Function iS_NewEntryCtrl(ctrlName) : ButtonControl
	String ctrlName
	
	wave/T M_ColourDataBase
	ReDimension/N=(DimSize(M_ColourDataBase,0)+1, -1) M_ColourDataBase
	iS_ColourDBPanel(M_ColourDataBase)
End

Function iS_SyncDBCtrl(ctrlName) : ButtonControl
	String ctrlName
	
	 iS_syncColourDataBase()
End


function iS_syncColourDataBase()
	wave/T/Z M_Model=  root:Packages:iSpectra:M_Model
	wave/T/Z M_ColourDataBase
		
	if(!WaveExists(M_Model) || !WaveExists(M_ColourDataBase))
		DoAlert 0, "WaveExists() Error."
		return 0
	endif
	
	variable nSIPhases= DimSize(M_Model, 0)
	variable nDBPhases = DimSize(M_ColourDataBase, 0)
	variable i
	
	// make a string from DB names
	Make/FREE/N=(nDBPhases) W_DBNames
	string TheNames=""
	for (i=0; i<nDBPhases; i+=1)
		TheNames += (M_ColourDataBase[i][3] +";")
	endfor
		
	
	for (i=0; i<nSIPhases; i+=1)
		string SIPhase = M_Model[i][%Phase]
		
		if(strlen(SIPhase)==0)
			continue
		endif
	
		variable pos = WhichListItem(SIPhase, TheNames, ";", 0, 0)
		
		if(pos != -1)
			M_Model[i][%R] = M_ColourDataBase[pos][0]
			M_Model[i][%G] = M_ColourDataBase[pos][1]
			M_Model[i][%B] = M_ColourDataBase[pos][2]
		endif
	endfor
	
	iS_OpenGroupsOnGrey()
	iS_iSpectraGroupPanel()		
end
